%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeEstimatedFirstStage.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: March 5, 2016
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% Note: The variable linCombPdfHatRescaled is poorly named because both 
% outputs are actually rescaled, but the second one is also truncated so
% that it doesn't take negative values.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [linCombPdfHat, linCombPdfHatRescaled] = ...
    computeEstimatedFirstStage(GridParams,GridOutputs,...
        EstimationParams,X,Y,computingMISEs,dgpGridChoice)
    
if computingMISEs == 1
    
    undersmooth = 0;
    
    useRescaledBandwidths = 0;
    % For the first step, where we compute the MISE for various bandwidths, 
    % it should be set equal to zero.
    
elseif computingMISEs == 0
    
    undersmooth = 1; % Indicator for whether we undersmooth
    
    % undersmoothPercentage gives us a simple way to tradeoff bias and
    % variance, without having to adjust each first stage bandwidth
    % separately. You can run the code with several different percentages
    % and see this tradeoff. The choice between the various undersmoothing
    % amounts simply depends on which you care about more: bias or
    % variance. Here I have chosen them so that neither is too extreme
    % (although this is of course a somewhat subjective judgement)
    
    if  dgpGridChoice == 1 || dgpGridChoice == 3
        undersmoothPercentage = 1;
    else
        undersmoothPercentage = 0.7;
    end
    
    useRescaledBandwidths = 1;
    % This parameter should only be turned on (=1) AFTER we have computed
    % optimal bandwidths and we are running the main simulation, for FIXED
    % bandwidths.

end
    
%% 1. Set parameters
nTgrid = GridParams.nTgrid;
nSgrid = GridParams.nSgrid;
nControlVarsGrid = GridParams.nControlVarsGrid;

r = EstimationParams.r;
lambda = EstimationParams.lambda;
clambda = EstimationParams.clambda;

% Numerical integration 'weights'/'steps'/'deltas'
allVarsSteps(:,1) = GridOutputs.SgridSteps'; % nTgrid-by-1
allVarsSteps(:,2) = GridOutputs.controlVarsGridSteps'; % nTgrid-by-1

linCombPdfHat = zeros(nTgrid,nSgrid);
linCombPdfHatRescaled = zeros(nTgrid,nSgrid);

%% 2. For each (t1,t2), compute the estimated linear combination density
for it = 1:nTgrid
    display(it);
    
    % Pick the optimal bandwidth for this particular (t1,t2) point
    if useRescaledBandwidths == 0
        h = EstimationParams.bandwidthGrid{it};        
    elseif useRescaledBandwidths == 1
        h = EstimationParams.bandwidthGrid_resc{it};       
    end
    
    if undersmooth == 1
        h = undersmoothPercentage*h;
    end
    
    allVarsGrid(:,:) = GridOutputs.allVarsGrid(:,:,it);
    
    linCombPdfHatVec = nan(1,nSgrid);
    linCombPdfHatRescaledVec = nan(1,nSgrid);

    %% Compute estimator

    CurrentEstimParams.h = h;
    CurrentEstimParams.lambda = lambda;
    CurrentEstimParams.r = r;
    CurrentEstimParams.clambda = clambda;
    CurrentEstimParams.tau = 0;
    CurrentEstimParams.XcolumnWithIntercept = 3;

    % (1) the entire joint pdf of random coefficients
    fABhat = ...
    computeRadonEstimator(X,Y(:,it),allVarsGrid,CurrentEstimParams);

    % Normalize so that integrates to 1
    integral = sum(fABhat)*allVarsSteps(it,1)*allVarsSteps(it,2);        
    fABhat = fABhat/integral; % (nSgrid * nControlVarsGrid)-by-1
    clear integral

    fABhatReshaped = reshape(fABhat,[nControlVarsGrid,nSgrid]);

    fpi22 = sum(fABhatReshaped,1)*allVarsSteps(it,2);
    fpi22 = squeeze(fpi22); 
    fpi22 = fpi22';

    % (2) the pdf of only the coefficients on the first column of X
    linCombPdfHatVec(1,:) = fpi22;
        % 1-by-Na summing along rows = over values for b

    % (3) the truncated version of (2)
    fpi22 = max(fpi22,0);
    % Note: only linCombPdfHatRescaled uses this

    integral = sum(fpi22)*allVarsSteps(it,1);
    
    linCombPdfHatRescaledVec(1,:) = fpi22/integral;
        % 1-by-Na summing along rows = over values for b
    
    %% Save estimates
    
    linCombPdfHat(it,:) = linCombPdfHatVec(1,:);
    clear linCombPdfHatVec;

    linCombPdfHatRescaled(it,:) = linCombPdfHatRescaledVec(1,:);
    clear linCombPdfHatRescaledVec;
       
end % loop over (t1,t2) values

end % function
