%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeEstimatedFirstStageEmpirical.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: May 14, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [linCombPdfHat, linCombPdfHatRescaled, ...
    optimalBandwidth, optimalBandwidthRescaled] = ...
    computeEstimatedFirstStageEmpirical(GridParams,GridOutputs,...
        EstimationParams,linCombPdfTrue,X,Y)
%%
nTgrid = GridParams.nTgrid;
nSgrid = GridParams.nSgrid;
nControlVarsGrid = GridParams.nControlVarsGrid;

radonBandwidths = EstimationParams.radonBandwidths;
r = EstimationParams.r;
lambda = EstimationParams.lambda;
clambda = EstimationParams.clambda;

% Numerical integration 'weights'/'steps'/'deltas'
allVarsSteps(:,1) = GridOutputs.SgridSteps'; % nTgrid-by-1
allVarsSteps(:,2) = GridOutputs.controlVarsGridSteps'; % nTgrid-by-1

optimalBandwidth = zeros(nTgrid,1);
optimalBandwidthRescaled = zeros(nTgrid,1);

linCombPdfHat = zeros(nTgrid,nSgrid);
linCombPdfHatRescaled = zeros(nTgrid,nSgrid);

%% For each (t1,t2), compute the estimated linear combination density

for it = 1:nTgrid
    
    if mod(it,250) == 0
        display(it);
    end

    % Compute scaled bandwidths
    % This is for the empirical section.
    % The goal here is to set a SINGLE bandwidth for all Tgrid points, by
    % picking a master bandwidth and then just scaling it according to the
    % length of the Tgrid domain.
    %
    % Basic idea: Let R_baseline denote the range for a baseline domain.
    % Let h_baseline denote the optimal bandwidth for that range. Let R_new
    % denote the range for the new estimate we want. Then the new bandwidth
    % is
    %
    %       h_new = (R_new / R_baseline) * h_baseline
    %
    radonBandwidths = ...
       (EstimationParams.masterBandwidth)...
            *(GridOutputs.pi2linCombRange(it))...
            /EstimationParams.masterBandwidthBaselineRange;

    allVarsGrid(:,:) = GridOutputs.allVarsGrid(:,:,it);
    
    linCombPdfHatVec = zeros(length(radonBandwidths),nSgrid);
    linCombPdfHatRescaledVec = zeros(length(radonBandwidths),nSgrid);
   
    %% For several different choices of bandwidths
    for ih = 1:length(radonBandwidths)
        
        h = radonBandwidths(ih);

        %% Compute estimator
        
        CurrentEstimParams.h = h;
        CurrentEstimParams.lambda = lambda;
        CurrentEstimParams.r = r;
        CurrentEstimParams.clambda = clambda;
        CurrentEstimParams.tau = 0;
        CurrentEstimParams.XcolumnWithIntercept = 3;

        % (1) the entire joint pdf of random coefficients
        fABhat = ...
        computeRadonEstimator(X,Y(:,it),allVarsGrid,CurrentEstimParams);
        
        % Normalize so that integrates to 1
        integral = sum(fABhat)*allVarsSteps(it,1)*allVarsSteps(it,2);        
        fABhat = fABhat/integral; % (nSgrid * nControlVarsGrid)-by-1
        clear integral
        
        fABhatReshaped = reshape(fABhat,[nControlVarsGrid,nSgrid]);
        
        fpi22 = sum(fABhatReshaped,1)*allVarsSteps(it,2);
        fpi22 = squeeze(fpi22); 
        fpi22 = fpi22';

        % (2) the pdf of only the coefficients on the first column of X
        linCombPdfHatVec(ih,:) = fpi22; % 1-by-Na summing along rows = over values for b

        % (3) the truncated version of (2)
        fpi22 = max(fpi22,0);
        
        integral = sum(fpi22)*allVarsSteps(it,1);
        linCombPdfHatRescaledVec(ih,:) = fpi22/integral; % 1-by-Na summing along rows = over values for b

    end
    
    index = 1;
    optimalBandwidth(it,1) = radonBandwidths(index);
    linCombPdfHat(it,:) = linCombPdfHatVec(index,:);
    clear linCombPdfHatVec

    optimalBandwidthRescaled(it,1) = radonBandwidths(index);
    linCombPdfHatRescaled(it,:) = linCombPdfHatRescaledVec(index,:);
    clear linCombPdfHatRescaledVec

end

end
