%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeGamma2TSLS.m
% 
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Sept 27, 2012
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Estimates the reduced form mean regressions
%
% INPUTS:
% Z1 is a (n by L) vector of person 1's value of variables
% Z2 is a (n by L) vector of person 2's value of variables
% These variables are excluded: Z2 does not affect Z1 and vice versa
% The instrument must be the first column of these matrices
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function gamma2hatTSLS = computeGamma2TSLS(Y,Z1,Z2,X)

sampleSize = size(Y,1);

Y1 = Y(:,1);
Y2 = Y(:,2);

%% First stage: Regress Y1 on all exogenous variables
depVarMat1 = [ones(sampleSize,1) Z1 Z2 X];
pi1hat = (depVarMat1'*depVarMat1)\depVarMat1'*Y1;

% Get first stage residuals
y1resid = (depVarMat1*pi1hat);

%% Second stage: Regress Y1 on the Y2 residuals
depVarMat2 = [ones(sampleSize,1) y1resid Z2 X]; 
% Note the exclusion restriction is imposed---Z1 does not enter the
% structural equation for Y2

TSLSestimate = (depVarMat2'*depVarMat2)\depVarMat2'*Y2;

% Order of output:
% Constant
% Endogenous variable (gamma2)
% Z1 variables
% X variables

gamma2hatTSLS = TSLSestimate(2);

end