%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeMCresults.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Feb 2, 2016
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Read in the results from the MC simulations and compute 
% various performance statistics
%
% INPUT: Sample size, dgpChoice, and the number of simulations
%
% normalZ = 1 if you want to use normally distributed instruments
%         = 0 if you want Cauchy distributed instruments
%
% OUTPUT: MCresults, a structure with all of the Monte Carlo simulation
% results
%
% VARIABLES USED:
% Zgrid
% GridParams.nZgrid
% fgamma2Hat
% fgamma2True
%
% Variables it would be useful to keep around:
%
% optimalBandwidth
% linCombPdfHat
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function computeMCresults(sampleSize,dgpChoice,nSimulations,normalZ)
%% 0. Load TSLS parameter estimates

gamma2hatTSLSvector = [];

for i=1:nSimulations
    simulationNumber = i;
    
    try
        load(sprintf('../estimatesTSLS/MC_TSLS_n%d_dgp%d_nZ%d_sim%d.mat',...
            sampleSize,dgpChoice,normalZ,simulationNumber));
        
        gamma2hatTSLSvector = [gamma2hatTSLSvector TSLSestimate];
                      
    catch
        disp(sprintf('Did not load TSLS simulation dataset number %d.',...
            simulationNumber)); 
    end
end


%% 1. Load nonparametric parameter estimates
% Important note: sampleSize and dgpChoice are function input parameters,
% but they will be overwritten when we load the first dataset. This doesn't
% matter, however, because they will be overwritten with the same values!

fgamma2HatMatrix = [];
% Different columns = different simulation runs
% Different rows = different values of the grid we are evaluating f_gamma2
% at

nSimDatasets = 0;
for i=1:nSimulations
    simulationNumber = i;
    
    try            
        load(sprintf('../output_secondStage/MCestimates_n%d_dgp%d_nZ%d_sim%d.mat',...
                            sampleSize,dgpChoice,normalZ,simulationNumber));

        % Only for when all the simulation datasets exist
        % fgamma2HatMatrix(:,simulationNumber) = fgamma2Hat;
        fgamma2HatMatrix = [fgamma2HatMatrix fgamma2Hat];

        nSimDatasets = nSimDatasets + 1;
    catch
        %{
        disp(sprintf('Did not load simulation dataset number %d.',...
            simulationNumber)); 
        %}
        disp(sprintf('%d',...
            simulationNumber)); 
    end
end

% Note that at this point we now have all the parameters loaded from the
% last simulation run. These parameters, such as the true pdf of gamma2
% will be used below. These parameters are the same for all simulation
% runs (that's the point).

%% 1b. Truncate and rescale our estimated densities
Zstep = range(Zgrid)/(GridParams.nZgrid-1);
% This step is used here and several places below

for i=1:nSimDatasets
    
    fgamma2HatMatrix(:,i) = max(fgamma2HatMatrix(:,i),0);
    
    integral = sum(fgamma2HatMatrix(:,i))*Zstep;
    fgamma2HatMatrix(:,i) = fgamma2HatMatrix(:,i)/integral;
end

%% 2. Compute true parameter values
% This part is similar to the first version of my simulations with sieves

theta = 0.95;

if dgpChoice == 1

    mu_gamma     = 0.4;
    sigma_gamma  = 0.05;

    alph = (0-mu_gamma)/sigma_gamma;
    bet = (1-mu_gamma)/sigma_gamma;
    trunc_denom = normcdf(bet,0,1)-normcdf(alph,0,1);
    
    mean_gamma_true = mu_gamma + sigma_gamma*(normpdf(alph,0,1) ...
        -normpdf(bet,0,1))/trunc_denom;

    mean_gamma_true = theta*mean_gamma_true;
    
    fgamma2TrueVar = (sigma_gamma^2)*(1 ...
        + (alph*normpdf(alph,0,1) - bet*normpdf(bet,0,1))/trunc_denom ...
        - ((normpdf(alph,0,1) - normpdf(bet,0,1))/trunc_denom)^2 );

    % Because var(aX) = a^2 var(X)
    fgamma2TrueVar = (theta^2)*fgamma2TrueVar;  
    
    % Note: Let Q_{aX}(tau) be the tau-th quantile of aX, where a > 0. That is,
    % 
    %   tau = F_{aX}[ Q_{aX}(tau) ]
    %
    % Then since F_{aX}(t) = P(aX \leq t) = P(X \leq t/a) = F_X(t/a)
    % we have
    %
    % Q_{aX}(tau) = a * F_X^{-1}(tau) = a * Q_X(tau)
    %
    % which is just quantile equivariance!

    % From Mathematica: theta*Quantile[TruncatedDistribution[{0, 1}, 
    %                                  NormalDistribution[0.4, 0.05]], 0.5]
    fgamma2TrueQuantile50 = 0.38;    
    fgamma2TrueIQR = 0.06407652626862771;
    
    clear alph bet;

elseif dgpChoice == 3
    
    alpha_gamma  = 6;
    beta_gamma   = 3;

    mean_gamma_true = alpha_gamma/(alpha_gamma + beta_gamma);
    
    mean_gamma_true = theta*mean_gamma_true;
    
    fgamma2TrueVar = (alpha_gamma*beta_gamma)...
        /((alpha_gamma+beta_gamma)^2 * (alpha_gamma+beta_gamma+1));
    
    fgamma2TrueVar = (theta^2)*fgamma2TrueVar;
    
    % From Mathematica: theta*Quantile[BetaDistribution[6, 3], 0.75]
    fgamma2TrueQuantile25 = 0.5384550919752723;
    fgamma2TrueQuantile50 = 0.6455069810673976;
    fgamma2TrueQuantile75 = 0.7404517178394893;
    
    fgamma2TrueIQR = 0.201996625864217;

elseif dgpChoice == 5 || dgpChoice == 7

    alpha_gamma = 0.3;
    mu_gamma = [0.3; 0.7];
    sigma_gamma = [.05, .06];
    
    mean_gamma_true = ...
        alpha_gamma * mu_gamma(1) + (1-alpha_gamma) * mu_gamma(2);
    
    mean_gamma_true = theta*mean_gamma_true;   
    
    % From wikipedia: https://en.wikipedia.org/wiki/Mixture_distribution
    %{ 
    % Mean of mixture of normals:
    \operatorname{E}[X] = \mu = \sum_{i = 1}^n w_i \mu_i
    
    % Variance of mixture of normals:
    \operatorname{E}[(X - \mu)^2] = \sigma^2 = \sum_{i=1}^n w_i(\mu_i^{2} + \sigma_i^2) - \mu^2 . 
    %}
    
    fgamma2TrueVar = alpha_gamma*(mu_gamma(1)^2 + sigma_gamma(1)^2) + ...
        (1-alpha_gamma)*(mu_gamma(2)^2 + sigma_gamma(2)^2) - ...
        mean_gamma_true;
    
    fgamma2TrueVar = (theta^2)*fgamma2TrueVar;
    
    % From Mathematica: theta*Quantile[MixtureDistribution[{0.3,0.7}, {NormalDistribution[0.3, 0.05],NormalDistribution[0.7, 0.06]}], 0.75]
    fgamma2TrueQuantile25 = 0.330953;
    fgamma2TrueQuantile50 = 0.632741;
    fgamma2TrueQuantile75 = 0.685868;
    
    fgamma2TrueIQR = 0.354916;

end

%% 3. Compute estimated means of E(gamma_2)

fgamma2HatMeans = nan(nSimDatasets,1);

for i=1:nSimDatasets
    
    fgammat2HatCurrent = fgamma2HatMatrix(:,i);
    
    fgamma2HatMeans(i) = sum(Zgrid.*fgammat2HatCurrent)*Zstep;
    
end

% Nonparametric estimates
mean_gamma_medbias_nonpara = median(fgamma2HatMeans - mean_gamma_true);

mean_gamma_nonpara_IQR = ...
    quantile(fgamma2HatMeans - mean_gamma_true,0.75) - ...
    quantile(fgamma2HatMeans - mean_gamma_true,0.25);

% TSLS estimates
mean_gamma_medbias_TSLS = median(gamma2hatTSLSvector - mean_gamma_true);

mean_gamma_TSLS_IQR = ...
    quantile(gamma2hatTSLSvector - mean_gamma_true,0.75) - ...
    quantile(gamma2hatTSLSvector - mean_gamma_true,0.25);

%% 4. Compute MISE

fgamma2HatISEvec = zeros(nSimDatasets,1);

for i=1:nSimDatasets
    
    fgammat2HatCurrent = fgamma2HatMatrix(:,i);
    
    squaredDistance = (fgammat2HatCurrent - fgamma2True).^2;
    fgamma2HatISEvec(i) = sum(squaredDistance)*Zstep;
    
end

median_ISE = median(fgamma2HatISEvec);

MISE_IQR = ...
    quantile(fgamma2HatISEvec,0.75) - ...
    quantile(fgamma2HatISEvec,0.25);

%% 5. Find upper and lower Monte Carlo confidence bands

fgamma2HatMatrix_upperBand = nan(size(fgamma2HatMatrix,1),1);
fgamma2HatMatrix_lowerBand = nan(size(fgamma2HatMatrix,1),1);

for vi = 1:size(fgamma2HatMatrix,1);
    
    fgamma2HatMatrix_upperBand(vi) = ...
        quantile(fgamma2HatMatrix(vi,:),0.975);
    
    fgamma2HatMatrix_lowerBand(vi) = ...
        quantile(fgamma2HatMatrix(vi,:),0.025);
    
end

%% 6. Find the simulation run with the median IMSE

[val ind] = mymedian(fgamma2HatISEvec);
median_ISE_simRun = ind(1); % There may be two values closest to the 
                            % median, when there are an even number of
                            % simulations. I just arbitrarily take one of
                            % them.

%% 7. Plot the estimate for the simulation at the median

fgammat2HatMedianISE = fgamma2HatMatrix(:,median_ISE_simRun);

printDirectory = sprintf('../images/MCmedianISEpdf_n%d_dgp%d_nZ%d_sim%d.eps',...
    sampleSize,dgpChoice,normalZ,median_ISE_simRun);

plotFinalPdfEstimateVsTruth(GridParams,DgpParams,...
    fgamma2True,fgammat2HatMedianISE,Zgrid,dgpChoice,printDirectory,...
    fgamma2HatMatrix_lowerBand,fgamma2HatMatrix_upperBand);

%% 8. Compute the median and IQR (i.e., the 25,50,75th quantiles)
nQuantileGrid = 50000;

fgamma2HatMedian = nan(nSimDatasets,1);
fgamma2HatIQR = nan(nSimDatasets,1);

for i=1:nSimDatasets
    
    fgammat2HatCurrent = fgamma2HatMatrix(:,i);
    
    fgamma2HatMedian(i) = computeQuantile(nQuantileGrid,Zgrid,fgammat2HatCurrent,0.5,DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);
    
    quantile75 = computeQuantile(nQuantileGrid,Zgrid,fgammat2HatCurrent,0.75,DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);
    quantile25 = computeQuantile(nQuantileGrid,Zgrid,fgammat2HatCurrent,0.25,DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);
    
    fgamma2HatIQR(i) = quantile75 - quantile25;
    
end

median_gamma_medbias_nonpara = median(fgamma2HatMedian - fgamma2TrueQuantile50);

median_gamma_nonpara_IQR = ...
    quantile(fgamma2HatMedian - fgamma2TrueQuantile50,0.75) - ...
    quantile(fgamma2HatMedian - fgamma2TrueQuantile50,0.25);

IQR_gamma_medbias_nonpara = median(fgamma2HatIQR - fgamma2TrueIQR);

IQR_gamma_nonpara_IQR = ...
    quantile(fgamma2HatIQR - fgamma2TrueIQR,0.75) - ...
    quantile(fgamma2HatIQR - fgamma2TrueIQR,0.25);


%% 9. Save results

% MEAN ESTIMATES USING NONPARAMETRIC EST

MCresults.mean_gamma_medbias_nonpara = mean_gamma_medbias_nonpara;
MCresults.mean_gamma_nonpara_IQR = mean_gamma_nonpara_IQR;

MCresults.mean_gamma_medbias_TSLS = mean_gamma_medbias_TSLS;
MCresults.mean_gamma_TSLS_IQR = mean_gamma_TSLS_IQR;

% MISE ESTIMATES

MCresults.median_ISE = median_ISE;
MCresults.MISE_IQR = MISE_IQR;

% MEDIAN ESTIMATES

MCresults.median_gamma_medbias_nonpara = median_gamma_medbias_nonpara;
MCresults.median_gamma_nonpara_IQR = median_gamma_nonpara_IQR;

% IQR ESTIMATES

MCresults.IQR_gamma_medbias_nonpara = IQR_gamma_medbias_nonpara;
MCresults.IQR_gamma_nonpara_IQR = IQR_gamma_nonpara_IQR;


save(sprintf('../output_MCresults/MCresults_n%d_dgp%d_nZ%d.mat',...
    sampleSize,dgpChoice,normalZ),'MCresults');

end