%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeMISE.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: April 23, 2016
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function computeMISE(...
    sampleSize,dgpChoice,nSimulations_MISE,normalZ,...
    GridParams,EstimationParams,GridOutputs,linCombPdfTrue,...
    radonBandwidth,radonBandwidthIndex,useWeightedISE)

ISE = zeros(GridParams.nTgrid,nSimulations_MISE);
ISErescaled = zeros(GridParams.nTgrid,nSimulations_MISE);

for MISEsimul = 1:nSimulations_MISE

    display(MISEsimul);

    %% Load simulated data

    % This block of code ensures that the data will load, even if it is
    % being accessed by multiple instances in parallel at the same time.
    count = 0;
    err_count = 0;
    while count == err_count
        try
            load(sprintf('../datasets_MISE/MCdataset_n%d_dgp%d_nZ%d_sim%d.mat',...
                sampleSize,dgpChoice,normalZ,MISEsimul));
        catch MyErr
            err_count = err_count + 1;
        end
        count = count + 1;
        if count > 1000
            display('Data failed to load.');
            return;
        end
    end
    display(sprintf('Data loaded after %d tries....',count));


    X = [Z ones(sampleSize,1)]; %sampleSize-by-3
    
    % Y is loaded here too, sampleSize-by-2
        
    % Compute linear combinations of the outcome variables
    % We will pass these to the estimator
    YlinComb = Y * (GridOutputs.Tgrid'); % sampleSize-by-nTgrid
        % Y is sampleSize-by-2
        % GridOutputs.Tgrid is nTgrid-by-2
        % so GridOutputs.Tgrid' is 2-by-nTgrid
        %
        % Y = ( Y1 Y2 )
        %     (  .  . )
        %        .
        %        .
        %
        % GridOutputs.Tgrid = (t1 t2)
        %                       .  .
        %                       .  .
        %
        % Y * (GridOutputs.Tgrid') = (Y1 Y2) ( t1 . .
        %                              .  .  ( t2 . .
        %
        % So Y(:,it) is a sampleSize-by-1 vector of
        %       t1*Y1 + t2*Y2
        % where (t1,t2) are the it'th row of Tgrid

    %% Estimate
    
    % Fix all (t1,t2) bandwidths to be the same, just for the current
    % purpose of computing the MISE for a fixed bandwidth
    for it = 1:GridParams.nTgrid
        EstimationParams.bandwidthGrid{it} = radonBandwidth;
    end
    
    computingMISEs = 1;
    
    % Obtain estimate for this single bandwidth
    [linCombPdfHat, linCombPdfHatRescaled] = ...
    computeEstimatedFirstStage(GridParams,GridOutputs,...
        EstimationParams,X,YlinComb,computingMISEs,[]);

    % Compute the ISE for this bandwidth    
    [ISE(:,MISEsimul), ISErescaled(:,MISEsimul)] = ...
        computeEstimatedFirstStageISE(...
        linCombPdfHat,linCombPdfHatRescaled,...
        linCombPdfTrue,GridParams,useWeightedISE);

end % loop over MISE simulations

thisMISE = sum(ISE,2)/nSimulations_MISE; % nTgrid-by-1
thisMISErescaled = sum(ISErescaled,2)/nSimulations_MISE; % nTgrid-by-1
% These should be the same vector!

% Save MISE
    display('Saving MISEs...');
    try
    save(sprintf('../output_bandwidths/MISEs_n%d_dgp%d_nZ%d_h%d.mat',...
        sampleSize,dgpChoice,normalZ,radonBandwidthIndex),...
        'thisMISE','thisMISErescaled'); 
    display('MISE dataset save successful.');
    catch
        warning('MISE dataset not saved.');
    end
    
end % function