%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeRadonEstimator.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Nov 8, 2014
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Implements the Hoderlein, Klemela, Mammen (2010) estimator 
% of the distribution of random coefficients in a single equation linear
% random coefficient model.
%
% INPUTS: 
%   X = vector of covariates, including an intercept term if desired
%    (sampleSize-by-covariateDimension)
%
%   Y = vector of outcome variables
%    (sampleSize-by-1)
%
%   ABgridvec = vector of points at which we want to compute the estimated
%   density function
%    (nABgrid-by-1)
%
%    For example, suppose that there is an intercept and a scalar
%    regressor. Then there is a two-dimensional vector of random
%    coefficients, say, (A,B). If elsewhere we specified a grid of points
%    for A and a grid of points for B, then taking the tensor product of
%    these grids yields
%
%           a1,b1  a2,b1  a2,b1, ... , an,b1
%           a1,b2  a2,b2  a3,b2, ... , an,b2
%           a1,b3  a2,b3  a2,b3, ... , an,b3
%           ...     ...     ...
%           a1,bn  a2,bn  a2,bn, ... , an,bn
%
%    which we would then reshape to be a single column vector of all of
%    these pairs. That column vector is ABgridvec
%
%    EstimationParams = a structure containing the tuning parameters
%
% OUTPUTS: 
%   fABhat = the density of random coefficients evaluated at the points
%   contained in ABgridvec
%
%   nObsDeletedByTrimming = the number of observations that were deleted in
%   the trimming step
%
% FUNCTIONS CALLED: 
%   kernelG.m
%   kernelK.m
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [fABhat, nObsDeletedByTrimming] = ...
    computeRadonEstimator(X,Y,ABgridvec,EstimationParams)
%% Load estimation parameters

lambda = EstimationParams.lambda;
r = EstimationParams.r;
h = EstimationParams.h;
tau = EstimationParams.tau;
clambda = EstimationParams.clambda;
XcolumnWithIntercept = EstimationParams.XcolumnWithIntercept;

% covariateDimension includes any intercept terms
sampleSize = size(X,1);
covariateDimension = size(X,2);
nABgrid = size(ABgridvec,1);

%% Transform the data to spherical

normOfX = sqrt(sum(X.^2,2)); % sampleSize-by-1
U = Y./normOfX; % sampleSize-by-1

normOfX = normOfX*ones(1,covariateDimension); % sampleSize-by-covariateDimension
S = X./normOfX; % sampleSize-by-covariateDimension

clear normOfX

%% Trimming

nObsDeletedByTrimming = 0;

if tau > 0
    
    % This should reference whatever component of X is the intercept
    indicTrimming = (S(:,XcolumnWithIntercept) < tau);
    
    nObsDeletedByTrimming = sum(indicTrimming);
    
    S(indicTrimming,:) = [];
    U(indicTrimming,:) = [];
    
    sampleSize = sampleSize - nObsDeletedByTrimming;
end

%% Denominator = Estimated density of S data
% This part only depends on the (spherical) data. No grid points, just the
% constant clambda and the bandwidth lambda.
% Here we want the density fShat evaluated at each data point S_i.
% So eventually we want a sampleSize-by-1 vector.

% 1. Call kernel G
KG = kernelG(S,S,lambda); % sampleSize-by-sampleSize

% 2. Numerical integration
denom = clambda*sum(KG,2)/sampleSize; % sampleSize-by-1
% check denom should not be 0 to be able to divide
% ---actually, denom should always be >0, by the analytical formula
clear KG

%% Numerator = K_{r,h}
% First, get input S_i'b - U_i at which the kernel K will be evaluated

% S_i'b = S_i(1)*b(1) + S_i(2)*b(2)
Sb = zeros(sampleSize,nABgrid);
for id = 1:covariateDimension
    Sb = Sb + S(:,id)*ABgridvec(:,id)'; % sampleSize-by-nABgrid
end

% S_i'b - U_i
SbMinusU = Sb - U*ones(1,nABgrid); % sampleSize-by-nABgrid;
clear Sb

% The Radon kernel K_{r,h} is only coded analytically for these parameters
if r==1 && (covariateDimension==2 || covariateDimension==3)
    
    num = kernelK(SbMinusU,covariateDimension,r,h);
    
% Otherwise we have to compute K_{r,h} via numerical integration
else
    % Define grid from 0 to 1/h for numerical integration
    NTgrid = 300;
    Tstep = (1/h)/NTgrid;
    Tgrid = 0:Tstep:(1/h - Tstep);

    num = kernelK(SbMinusU,covariateDimension,r,h,Tgrid,Tstep);
    clear NTgrid Tstep Tgrid
end

%% Division and final sum

denom = denom*ones(1,nABgrid); % sampleSize-by-nABgrid
summand = num./denom; % sampleSize-by-nABgrid
fABhat = sum(summand/sampleSize,1)'; % nABgrid-by-1

end