%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeSecondStage.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Feb 17, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Implements step 4 of the Masten (2017) RC estimator; see
% section 4 of the paper
%
% INPUTS:
%
%   > fPi: matrix of evaluation of f_{Pi(t1,t2)} over a grid for (t1,t2).
%   Can be:
%   (1) fPitrue, approximation of the true fPi by numerical integration 
%   (output of truefPi.m);
%
%   (2) or fPihat, estimation of fPi by the Hoderlein procedure (output of
%   main_mc_SEM_tgrid.m)
%
%   > Tgrid: the grid for (t1,t2) used to get fPi above
%
% OUTPUT: 
%
% fgamma2Hat = matrix of size (nZgrid-by-1), estimation of f_{gamma2}.
% Element (i,1) in the matrix is the evaluation of f_{gamma2} at z_i, where
% z_i is ith grid point.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [fgamma2Hat, Zgrid] = ...
    computeSecondStage(GridParams,DgpParams,GridOutputs,linCombPdf)

% Load parameters
nSgrid = GridParams.nSgrid;
nZgrid = GridParams.nZgrid;
nTgrid = GridParams.nTgrid;
nVgrid = GridParams.nVgrid;
TgridLowerBound = GridParams.TgridLowerBound;
TgridUpperBound = GridParams.TgridUpperBound;
VgridLowerBound = GridParams.VgridLowerBound;
VgridUpperBound = GridParams.VgridUpperBound;
gammaLowerBound = DgpParams.gammaLowerBound;
gammaUpperBound = DgpParams.gammaUpperBound;

Tgrid = GridOutputs.Tgrid;
Sgrid = GridOutputs.Sgrid;
SgridSteps = GridOutputs.SgridSteps;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 0b. Compute grids
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Compute grid for the z's at which we evaluate our final function
% f_{gamma_2}(z) at.
ZgridLength = gammaUpperBound - gammaLowerBound;   % Length of the interval
Zstep = ZgridLength/(nZgrid-1);        % Step size
Zgrid = (gammaLowerBound:Zstep:gammaUpperBound)';   % Compute uniform grid

% Compute the grid for v's at which we evaluate f_{pi_{22},pi_{12}}(zv,v)
% in step 4.
VgridLength = VgridUpperBound - VgridLowerBound; % Length of the interval
Vstep = VgridLength/(nVgrid-1);            % Step size
Vgrid = (VgridLowerBound:Vstep:VgridUpperBound)'; % Compute uniform grid

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 1. Linear combination reduced form pdf
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This step is computed by another matlab file, and so here we just load
% the output from that file.
%
% There are two possibilities here:
% 1. We load the true, analytically computed pdf f_{\Pi(t_1,t_2)}(s).
% 2. We load an estimated pdf \hat{f}_{\Pi(t_1,t_2)}(s).
%
% This allows us to separate out the effect of sampling uncertainty /
% estimation error on the final pdf f_gamma from the effect of the
% numerical approximations inherent in the % transformation steps we
% perform below.
%
% Here are the parameters that we load:
% 1. fPitrue = nSgrid x nTgrid matrix of f_{\Pi(t_1,t_2)}(s) values
% Note that while (t1,t2) is a two-dimensional vector, we are indexing
% these 2d grid points by a single index.
%
% 2. Tgrid = nTgrid x 2 matrix of (t1,t2) grid points
%
% 3. Sgrid = nSgrid x nTgrid matrix of points (s) at which we evaluate
% integrals of f_{\Pi(t_1,t_2)}(s).
% 
% Note that this is a one-dimensional integrand. The support of 
% Pi(t_1,t_2) is bounded, but it changes with (t1,t2). Hence the grid of
% points s which we want to integrate f_{\Pi(t_1,t_2)}(s) over also changes
% with (t_1,t_2). Each column of the matrix Sgrid is a different grid of
% points s, one column for each choice of (t_1,t_2) grid.
%
% 4. We also load some other parameters used below:
% -numbers of grid points
% -step sizes
% -grid bounds
% -etc

% (LOADED ABOVE)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2a. Grid for integration: Sgrid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Sgrid is imported as input: f_Pi{(t1,t2)} are evaluated at those grid
% points S already.
% 
% (Sgrid is loaded above)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2b. Compute the characteristic function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Here we compute
%
% phi_{\pi_{12},\pi_{22}(t_1,t_2)
%  = int_{\R} exp(is) f_{\Pi(t_1,t_2)}(s) ds
%
% or the estimated version, depending on what we input in step 1.

% Exponential term
% exp(is) = cos(s) + i*sin(s)
exponentialTerm = cos(Sgrid) + 1i*sin(Sgrid); % nSgrid-by-nTgrid
clear Sgrid
    %% sum(sum(isnan(exponentialTerm))) >> aucun nan jusque la
    
% Form the integrand
integrand = exponentialTerm.*linCombPdf; % nSgrid-by-nTgrid
clear exponentialTerm linCombPdf 
    %B=linCombPdfHat;
    %sum(sum(isnan(B)))
    %sum(sum(isnan(integrand))) >> aucun nan jusque la
    
% Scale the integrand; for example, equation 9.1.2 of Judd (1998)
Sstepmat = ones(nSgrid,1)*SgridSteps; % nSgrid-by-nTgrid
integrand = integrand.*Sstepmat; % nSgrid-by-nTgrid
clear SgridSteps

% Form the estimated characteristic function
phi = sum(integrand,1); % 1-by-nTgrid
clear integrand

% VERY IMPORTANT: Use .' instead of '
% The regular ' performs the conjugate transpose, and hence flips the sign
% on the imaginary numbers, which we DO NOT want.
phi = phi.';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 3. Convert the characteristic function to the pdf
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following steps give us the pdf
%
%   f_{pi_{12},pi_{22}}(p1,p2)
%
% or the estimated version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 3a. Grid for evaluation: Pgrid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pgrid will be nVgrid*nVgrid by 2

% We want to get a grid of points (zv,v)

% This gives a grid of points zv where z varies over Zgrid and 
% V varies over Vgrid
ZVgrid = Vgrid*Zgrid'; % nVgrid-by-nZgrid
% So: in each column, a z is fixed, and as the rows change, z is
% multiplied by all the v elements.

ZVgrid = ZVgrid(:); % (nVgrid*nZgrid)-by-1

% When vectorizing, the jth column is stacked below the (j-1)th column.
% So z1 is fixed over the first nVgrid rows, and v changes over them;
% then z2 is fixed over the next nVgrid rows, and v loops again over
% them, etc.
VgridReplicated = Vgrid*ones(1,nZgrid); % nVgrid-by-nZgrid

VgridReplicated = VgridReplicated(:); % (nVgrid*nZgrid)-by-1

% WARNING: The order of columns in Pgrid is very important. the first
% column is the first argument of
%
%   f_{pi12,pi22}( . , . )
%
% while the second column is the second argument, in the ratio distribution
% integral. Since we want the ratio pi22/pi12, we evaluate the first
% argument at v and the second at zv. 
Pgrid = [VgridReplicated ZVgrid]; % (nVgrid*nZgrid)-by-2
clear VgridReplicated ZVgrid

nPgrid = nVgrid*nZgrid; % scalar

% To be clear:
%
% Pgrid = [
%  v1, v1 * z1
%  v2, v2 * z1
%  ...
%  vN, vN * z1
%
%  v1, v1 * z2
%  v2, v2 * z2
%  ...
%  vN, vN * z2
%
%  v1, v1 * zN
%  v2, v2 * zN
%  ...
%  vN, vN * zN
%  ]
%
% where I'm letting N = nVgrid = nZgrid
% (equal for illustrative purposes only)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 3b. Compute the integrand (exponential * characteristic function)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In this step we compute the integrand of the inverse Fourier transform

% First we get the argument of the exp( ) term:
%
% t1*p1 + t2*p2
%
% We need this scalar for each couple of pairs (t1,t2) and (p1,p2)

TPgrid = Tgrid(:,1)*Pgrid(:,1)' + Tgrid(:,2)*Pgrid(:,2)'; % nTgrid-by-nPgrid 
% clear Tgrid

% Next compute the exponential term
% 
% exp( -i [t1*p1 + t2*p2] )
%
% Here we use that cos is even and sin is odd, so 
% exp(-ix) = cos(x) - i sin(x)

exponentialTerm = cos(TPgrid) - 1i*sin(TPgrid); % nTgrid-by-nPgrid 
clear TPgrid

% Finally, multiply the exponential term by the characteristic function
% to get the integrand
%
% exp( -i [t1*p1 + t2*p2] ) * phi_{pi12,pi22}(t1,t2)

integrand = exponentialTerm.*(phi*ones(1,nPgrid)); % nTgrid-by-nPgrid
clear exponentialTerm

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 3c. Integrate over (t1,t2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In this step we compute the inverse Fourier transform, and thus get the
% pdf, by integrating the integrand we have just computed over (t1,t2)

% Compute the volume/area covered by Tgrid and divide by the number of
% grid points. For example, see equation 9.1.2 of Judd (1998)
Tdelta = ((TgridUpperBound-TgridLowerBound)^2)/nTgrid; % scalar

% Numerically integrate to get the joint pdf of pi_{12} and pi_{22},
%
% f_{pi12,pi22}(p1,p2)
%  = Real part of [ (1/(2pi)^2 int_{R^2} exp( -i [t1*p1 + t2*p2] ) 
%                                       * phi_{pi12,pi22}(t1,t2) dt1 dt2 ]
%

fpi12pi22 = sum(integrand*Tdelta,1)/((2*pi)^2); % 1-by-nPgrid
clear integrand Tdelta

% Take the real part only
fpi12pi22 = real(fpi12pi22); % 1-by-nPgrid

% Convert to a column vector
fpi12pi22 = fpi12pi22'; % nPgrid-by-1

% Reshape so that the rows are v's and the columns are z's
% That is, now the (i,j)th entry is
%
%   f_{pi12,pi22}(v_i, z_j v_i).
%
% Below we will integrate this over v for each fixed z.
% So we take our long column vector and reshape it so that the rows
% correspond to v's and the columns are z's
fpi12pi22 = reshape(fpi12pi22,nVgrid,nZgrid); % nVgrid-by-nZgrid

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 4. Get the pdf for the ratio random variable
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This step gives us f_{gamma_2}(z) where
%
%    gamma_2 = pi_{22} / pi_{12}
%
% is the ratio of the two random variables pi_{22} and pi_{12}.
% (or it gives us an estimated version)
%
% We apply the formula from Curtiss (1941):
%
% f_{pi22/pi11}(z) = int_{\R} |v| f_{pi12,pi22}(v,zv) dv.
%
% First we need to form the integrand. So we compute the absolute value of
% the v gird:
absV = abs(Vgrid); % nVgrid-by-1

% Replicate the column vector of absolute values of v grid points
% This is just nZgrid different copies of abs(Vgrid) placed side by side
absV = absV*ones(1,nZgrid); % nVgrid-by-nZgrid

% Form the integrand
%
% |v| f_{pi12,pi22}(v,zv)
%
integrand = fpi12pi22.*absV; % nVgrid-by-nZgrid

% Scale by the (scalar) step size
integrand = Vstep*integrand; % nVgrid-by-nZgrid

% Integrate over v
% That is, for each column (z values), sum over the rows (v values)
fgamma2Hat = sum(integrand,1); % 1-by-nZgrid
    fgamma2Hat=max(fgamma2Hat,0);
    nZgrid=length(Zgrid);
    ZgridStep=range(Zgrid)/(nZgrid-1);
    factor=sum(fgamma2Hat*ZgridStep);
    fgamma2Hat=fgamma2Hat/factor;
    
% Turn it into a column vector
fgamma2Hat = fgamma2Hat'; % nZgrid-by-1

end