%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeSphericalKernelConstant.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: March 10, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Computes the normalization constant c(lambda) as described
% in Hoderlein, Klemela, Mammen (2010)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function c = computeSphericalKernelConstant(lambda)
%%
% GRID FOR THE s's.
% Here, we want a grid over the unit circle. An easy way to proceed is to
% see that each point (s1,s2) on the unit cirle has polar coordinate
% (1,theta), where theta ranges from 0 to 2pi. Therefore, to get a grid, it
% we take a grid of angles over [0,2pi), and then find the corresponding
% cartesian coordinates (s1,s2) = (1.cos(theta), 1.sin(theta)).

Ngrid = 500;

%%%% for d=2
%
%thetastep = 2*pi/Ngrid;
%thetagrid = [0:thetastep:(2*pi-thetastep)]';
%Sgrid(:,1) = cos(thetagrid);
%Sgrid(:,2) = sin(thetagrid);
%
% Now, for the numerical integration, we need weights for the points. Here
% the weights are the same for all points, and equal to (circumference of
% the unit circle) / (number of grid points):
%Sdelta = 2*pi/Ngrid;
%
% Apply kernelG function
% In the expression of clambda, e can be ANY point of the unit sphere,
% so here, I take e=(0,1);
%evec = [0 1]; 

%%%% for d=3
% 
thetastep = 2*pi/Ngrid;
thetagrid = [0:thetastep:(2*pi-thetastep)]';

phistep = pi/Ngrid;
phigrid = [0:phistep:(pi-phistep)]';

thetamat = thetagrid*ones(1,Ngrid);
phimat = ones(Ngrid,1)*phigrid';

xmat = sin(thetamat).*cos(phimat);
ymat = sin(thetamat).*sin(phimat);
zmat = cos(thetamat);

Sgrid = [xmat(:) ymat(:) zmat(:)];
clear thetastep thetagrid xmat ymat zmat phimat phigrid phistep thetamat

Sdelta = 2*pi*pi/(Ngrid*Ngrid);
evec = [1 0 0];

integrand = kernelG(Sgrid,evec,lambda); % Ngrid-by-1

c = 1/(sum(integrand*Sdelta)); % Scalar

% check: scatter(thetagrid,integrand)
clear Sgrid Ngrid thetastep thetagrid Sdelta evec integrand

end