%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeTandSgridsEmpirical.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: May 14, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function GridOutputs = computeTandSgridsEmpirical(GridParams,DgpParams)

% Load parameters
nTgrid = GridParams.nTgrid;
TgridLowerBound = GridParams.TgridLowerBound;
TgridUpperBound = GridParams.TgridUpperBound;

covariateDimension = GridParams.covariateDimension;

nSgrid = GridParams.nSgrid;

nControlVarsGrid = GridParams.nControlVarsGrid;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% Compute the grid for (t1,t2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Our grid of (t_1,t_2) points is a square centered at zero:
% [ TgridLowerBound, TgridUpperBound ]^2
%
Tgrid = haltonseq(nTgrid,2); % nTgrid-by-2

Tgrid = TgridLowerBound + (TgridUpperBound-TgridLowerBound)*Tgrid; % nTgrid-by-2

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% For each (t1,t2), compute support of coeffs in t1*Y1 + t2*Y2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Here I specify the bounds on the ``conjectured' distribution of random
% coefficients. This conjectured distribution is used to compute the bounds
% on the grids for the linear combination of coefficients. The bounds on
% this conjectured distribution do NOT constrain the estimator's ability to
% estimate probability mass outside of these bounds.
gammaLB = 0;
gammaUB = 0.95;

%rng(5241,'twister')
[Z1_coeffs,Z2_coeffs,Intercepts] = ...
    generateRandomCoefficientsData(10000,[],DgpParams.betaZcoeff,...
    DgpParams.alpha,DgpParams.beta,DgpParams.mu,DgpParams.sigma,DgpParams.dist,gammaLB,gammaUB);
% Z1_coeffs = sampleSize-by-2 vector. Column 1 = equation 1

% First column is equation 1
% Second column is equation 2
pi12 = Z1_coeffs(:,1);
pi22 = Z1_coeffs(:,2);

pi13 = Z2_coeffs(:,1);
pi23 = Z2_coeffs(:,2);

pi11 = Intercepts(:,1);
pi21 = Intercepts(:,2);

t1 = Tgrid(:,1);
t2 = Tgrid(:,2);

upperQuantile = 0.995;
lowerQuantile = 0.005;    

% All of these are nTgrid-by-3

% Coeffs on Z1
% t1*pi12 + t2*pi22
piLinCombLowerBound(:,1) = quantile(t1*pi12' + t2*pi22',lowerQuantile,2)-1;
piLinCombUpperBound(:,1) = quantile(t1*pi12' + t2*pi22',upperQuantile,2)+1;

% Coeffs on Z2
% t1*pi13' + t2*pi23'
piLinCombLowerBound(:,2) = quantile(t1*pi13' + t2*pi23',lowerQuantile,2)-1;
piLinCombUpperBound(:,2) = quantile(t1*pi13' + t2*pi23',upperQuantile,2)+1;

% Intercepts
% t1*pi11' + t2*pi21'
piLinCombLowerBound(:,3) = quantile(t1*pi11' + t2*pi21',lowerQuantile,2)-1;
piLinCombUpperBound(:,3) = quantile(t1*pi11' + t2*pi21',upperQuantile,2)+1;

pi2linCombRange = piLinCombUpperBound(:,2) - piLinCombLowerBound(:,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Get Sgrid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For each pair (t1,t2), get the grid for s
%
% This is the grid of points at which we estimate f_{Pi(t1,t2)}(s)
% 
% This grid depends on (t1,t2) since the support of Pi(t1,t2) depends on t1
% and t2.

SgridSteps = zeros(1,nTgrid);
Sgrid = zeros(nSgrid,nTgrid);

for it = 1:nTgrid
    % SgridSteps(1,it) = ((piLinCombUpperBound(it,1)-piLinCombLowerBound(it,1))/(nSgrid-1));

    % Using linspace instead of colon here guarantees that there will
    % always be exactly nSgrid points. Using the colon does not guarantee
    % this.
    Sgrid(:,it) = linspace(piLinCombLowerBound(it,1),piLinCombUpperBound(it,1),nSgrid)'; % nSgrid-by-1

    SgridSteps(1,it) = range(Sgrid(:,it))/(nSgrid-1);
    
    % We will be dividing by the values in Sgrid(:,it).
    % (for computing the true linComb pdf)
    % Hence we need to make sure that they are nonzero.
    if sum((Sgrid(:,it)==0)) > 0
       error('S grid point is zero. Increase/decrease nSgrid by 1 and try again.');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Get controlVarsGrid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In the first stage, we estimate a random coefficinet model with
% covariateDimension different covariates (including the intercept).
% We are only interested in the coefficients on one of these covariates.
% Hence we need to integrate out the other dimensions. We get this grid
% here.
%
% IMPORTANT: piLinCombupperBound and piLinCombLowerBound must be ordered so
% that the first dimension is the one we want to integrate out
% 

preliminaryControlVarsGrid = ...
    haltonseq(nControlVarsGrid,covariateDimension-1);
% We're not done yet, because these grid points are on the unit cube. So we
% need to scale their dimensions up.

controlVarsGridSteps = zeros(1,nTgrid);
controlVarsGrid = zeros(nControlVarsGrid,covariateDimension-1);
allVarsGrid = zeros(nSgrid*nControlVarsGrid,covariateDimension,nTgrid);

for it = 1:nTgrid
    
    % Get the step size for the control variable grid
    rangeOfControls = (piLinCombUpperBound(it,:) - piLinCombLowerBound(it,:));
    rangeOfControls = rangeOfControls(2:covariateDimension);
    controlVarsGridSteps(1,it) = prod(rangeOfControls)/(nControlVarsGrid-1);
    clear range
    
    %% CRITICAL CHECK: THE COORDINATES OF THE CONTROLS HAVE TO MATCH UP
    % WITH THE COORDINATES OF PILINCOMB UPPER AND LOWER BOUNDS
    index = 1;
    for id = 2:covariateDimension
        
        % Rescale the preliminary grid: lower + (upper-lower)*[0,1]
        controlVarsGrid(:,index) = ...
            piLinCombLowerBound(it,id) ... 
            + (piLinCombUpperBound(it,id)-(piLinCombLowerBound(it,id)))...
            * preliminaryControlVarsGrid(:,index);
        
        index = index + 1;
        
    end
    clear index

    % Now we have two grids:
    % (1) Sgrid, the grid over the coordinate of interest, and 
    % (2) controlVarsGrid, the grid over the remaining covariateDimension
    %     covariates.
    %
    % We will combine these into a single matrix that is
    %
    % nSgrid*nControlVarsGrid-by-covariateDimension
    %
    % So the number of rows is the total number of grid points for the
    % entire domain, and the number of columns is the dimension of the
    % space
    
    % Sgrid(:,it) is nSgrid-by-1
    SgridReplicated = Sgrid(:,it)*ones(1,nControlVarsGrid);
    % nSgrid-by-nControlVarsGrid
    
    SgridReplicated = SgridReplicated'; % nControlVarsGrid-by-nSgrid
    
    allVarsGrid(:,1,it) = SgridReplicated(:);
    
    clear SgridReplicated
    
    controlVarsGridReplicated = repmat(controlVarsGrid,[nSgrid 1]);
    allVarsGrid(:,2:end,it) = controlVarsGridReplicated;
    clear controlVarsGridReplicated
    
    clear controlVarsGrid    
end
clear preliminaryControlVarsGrid

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Save the T and S grids
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

GridOutputs.controlVarsGridSteps = controlVarsGridSteps;
GridOutputs.allVarsGrid = allVarsGrid;
GridOutputs.Tgrid = Tgrid;
GridOutputs.Sgrid = Sgrid;
GridOutputs.SgridSteps = SgridSteps;
GridOutputs.piLinCombLowerBound = piLinCombLowerBound;
GridOutputs.piLinCombUpperBound = piLinCombUpperBound;
GridOutputs.pi2linCombRange = pi2linCombRange;

end