%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeTrueFirstStage.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Feb 17, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Computes the true pdf of f_{Pi2(t1,t2)} for a collection of 
% grid points (t1,t2). For each pair (t1j,t2j), f_{Pi2(t1j,t2j)} is 
% evaluated at grid points Sgrid(:,j) (the grid for S is chosen 
% specifically for the pair (t1j,t2j) as the support of f_{Pi2(t1,t2)} 
% changes with the pair (t1,t2)).
%
% INPUTS:
% Various Grid parameters and dgp parameters
%
% OUTPUT: 
% fPitrue = matrix of size (nSgrid-by-nTgrid). Element (i,j) in the
% matrix is the evaluation of f_{Pi2(t1j,t2j)} at s_{ij}, where s_{ij} is
% the ith grid point in the j-specific grid Sgrid(:,j).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function linCombPdf = computeTrueFirstStage(GridParams,DgpParams,GridOutputs)

%% 1. Load parameters
nTgrid = GridParams.nTgrid;
nSgrid = GridParams.nSgrid;
nG2grid = GridParams.nG2grid;

gammaLowerBound = DgpParams.gammaLowerBound;
gammaUpperBound = DgpParams.gammaUpperBound;

Tgrid = GridOutputs.Tgrid;
Sgrid = GridOutputs.Sgrid;

% First I need a grid for (t_1,t_2): input of this function. So i take the
% grid as given. Tgrid is nTgrid-by-2. So I have nTgrid different functions
% to evaluate.
%
% Second, I need a grid for s, at which each f_{Pi(t1,t2)} will be
% evaluated. Sgrid is nSgrid-by-1; Sgrid should cover the support of 
% f_{Pi(t1,t2)}. Sgrid is just ABgrid1 in main_mc_SEM_tgrid.m, so i
% reproduce here the code.
%
% Third, I need a grid for g2, to perform the integration. In theory, the
% integral is over R, so G2grid must be picked to that g2 spans the
% support of f_{gamma2}: so over [0,1].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2. Compute G2grid for integration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This should contain the support of gamma2. Do not set one of the bounds
% equal to zero, because we have to divide by g2 later and we don't want to
% divide by zero. The interval can, however, contain zero.
if gammaLowerBound == 0
    G2lowerBound = 1e-8;
else
    G2lowerBound = gammaLowerBound;
end
G2upperBound = gammaUpperBound;

G2step = (G2upperBound-G2lowerBound)/(nG2grid-1); % scalar

% exclude 0 because we divide by g2 below, so we just want to make sure the
% grid is bounded away from zero
if G2lowerBound < 0 && G2upperBound > 0
    G2grid = [(G2lowerBound:G2step:1e-8)'; (1e-8:G2step:G2upperBound)'];
else
    G2grid = (G2lowerBound:G2step:G2upperBound)'; % G2grid-by-1
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

linCombPdf = zeros(nSgrid,nTgrid);

for it = 1:nTgrid
    %% 3. Get the argument for f_{gamma1}

    %
    % 1/gamma2
    %
    inverseGamma2 = 1./G2grid; % nG2grid-by-1

    %
    % beta ( (1/gamma2)t1 + t2 )
    %
    argumentPiece1 = DgpParams.betaZcoeff*(inverseGamma2*Tgrid(it,1) ...
        + ones(nG2grid,1)*Tgrid(it,2)); % nG2grid-by-1

    %
    % beta ( (1/gamma2)t1 + t2 ) * (1/s)
    %
    argumentPiece2 = argumentPiece1*(1./Sgrid(:,it)'); % nG2grid-by-nSgrid

    %
    % (1/gamma2) - beta ( (1/gamma2)t1 + t2 ) * (1/s)
    %
    argumentPiece3 = ...
        inverseGamma2*ones(1,nSgrid) - argumentPiece2; % nG2grid-by-nSgrid

    clear argumentPiece2

    %
    % f_{gamma1}[ (1/gamma2) - beta ( (1/gamma2)t1 + t2 ) * (1/s) ]
    %
    f_gamma1 = ...
        computeUnivariatePdf(argumentPiece3,DgpParams); %nG2grid-by-nSgrid
             %debugging: grid=argumentPiece3;
    clear argumentPiece3

    %% 4. Get the full integrand and numerically integrate
    %
    % f_{gamma2}( gamma2 )
    %
    f_gamma2 = computeUnivariatePdf(G2grid,DgpParams); % nG2grid-by-1

    %
    % f_{gamma1}[ (1/gamma2) - beta ( (1/gamma2)t1 + t2 ) * (1/s) ]
    %   * beta ( (1/gamma2)t1 + t2 )
    %   * 1/(s^2)
    %   * f_{gamma2}( gamma2 )
    %
    integrand = f_gamma1.*(argumentPiece1*(1./(Sgrid(:,it).^2)'))...
        .*(f_gamma2*ones(1,nSgrid)); % nG2grid-by-nSgrid
    
    clear f_gamma1 f_gamma2 argumentPiece1

    % Numerical integration
    linCombPdf(:,it) = sum(integrand*G2step,1)'; % nSgrid-by-1   
    
    clear integrand

    %% 5. Adjust sign
    %
    % If s > 0 then the density is given by the expression above.
    % But if s < 0 then we need to multiply by -1.
    % 
    linCombPdf(:,it) = linCombPdf(:,it).*sign(Sgrid(:,it)); % nSgrid-by-1
    
    %% 6. Rescale
    % This is not always necessary; it is only necessary for dgps which
    % have computation issues at the boundary, such as normal(0,0.2)
    % truncated to be in [0,1]
    
    integral = sum(linCombPdf(:,it)*GridOutputs.SgridSteps(it));
    linCombPdf(:,it) = linCombPdf(:,it)/integral;
end

clear G2grid G2step

end