%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computeUnivariatePdf.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Jan 19, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: File written in relation to generateUnivariateData.m which
% generates draws from scalar random variables. The present file evaluates
% the pdf of these scalar random variables at a specified grid of points.
%
% INPUTS:
% > grid = grid points at which the pdf is to be evaluated
% > dist = indicator of the dgp: 
%   >> 1 = N(mu,sigma^2)
%   >> 2 = Truncated normal(mu,sigma^2) with range (alpha,beta)
%   >> 3 = Beta(alpha,beta)
% > alpha, beta, mu, sigma: parameters of the above dgps; use [] if unused
% parameter
%
% OUTPUT: pdf, with same size as input `grid'. Element (i,j) of pdf is 
% the evaluation of the pdf at element (i,j) of grid.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function true_pdf = computeUnivariatePdf(grid,DgpParams)

dist = DgpParams.dist;
alpha = DgpParams.alpha;
beta = DgpParams.beta;
mu = DgpParams.mu;
sigma = DgpParams.sigma;
shift = DgpParams.shift;
scale = DgpParams.scale;

% if X has pdf f_X(s) then bX + a has pdf
%
% (1/b) f_X[ (s-a)/b ]
%

if dist == 1
    
    % These are the points we will evaluated the density at.
    % They are standardized because we're applying the formula for bX+a
    % given above
    argument = (grid - shift)/scale;
    
    % The truncated normal distribution is obtained by taking the usual
    % normal pdf, truncating it from below at alpha and from above at beta,
    % and then rescaling what's left over to ensure it integrates to one.
    % Thus we take the N(mu,sigma^2) pdf,
    %
    % (1/sigma) phi[ (x-mu)/sigma ]
    %
    
    numerator = normpdf(argument,mu,sigma); % Ngrid-by-1
    
    % and divide ig by
    %
    % Phi(beta;mu,sigma) - Phi(alpha;mu,sigma)
    %
    
    denominator = normcdf(beta,mu,sigma) - normcdf(alpha,mu,sigma);
    
    % to ensure it integrates to one.
    
    true_pdf = numerator./denominator;  % Ngrid-by-1
    
    % We truncate the pdf to make sure it is zero outside of (alpha,beta)
    isInsideSupport = (argument > alpha).*(argument < beta);
    true_pdf = true_pdf.*isInsideSupport;
    
    % This gives us a truncated normal on (alpha,beta).
    % But for these simulations we will start always with
    % 
    % (alpha,beta) = (0,1)
    %
    % and then rescale and shift our random variable.
   
    % To obtain the pdf of the rescaled and shifted random variable, we
    % apply the formula mentioned above to get
    
    true_pdf = (1/scale)*true_pdf;
    
    % And that's it!

% Beta(alpha,beta)
elseif dist == 2
    
    % Here the steps are basically the same as above.
    argument = (grid - shift)/scale;
    
    true_pdf = (1/scale)*betapdf(argument,alpha,beta);

elseif dist == 3
    true_pdf=alpha*normpdf(grid,mu(1),sigma(1))+beta*normpdf(grid,mu(2),sigma(2));
 
end

end