%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generateSystemsData.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: March 29, 2016
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Generate data from a simple two-equation linear SEM
%
% This code generates data from a two-equation linear simultaneous
% equations model with random coefficients on the endogenous variables.
% 
% Structural system:
% 
% Y1 = gamma_1*Y2 + beta1*Z1 + U1
% Y2 = gamma_2*Y1 + beta2*Z2 + U2
%
% Reduced form:
%
% Y1 = beta1/(1-gamma1*gamma2)*Z1 
%       + (beta2*gamma1)/(1-gamma1*gamma2)*Z2
%       + (U1 * gamma1*U2)/(1-gamma1*gamma2)
%
% Y2 = (beta1*gamma2)/(1-gamma1*gamma2)*Z1 
%       + beta2/(1-gamma1*gamma2)*Z2 
%       + (U2 * gamma2*U1)/(1-gamma1*gamma2)
%
% The dgp generated here is for the pairs of best friends example.
% Hence, because the labeling of friend 1 vs friend 2 doesn't matter,
% we have that the variables across the equations have the same
% distribution. Thus we assume Z1~Z2, U1~U2, gamma1~gamma2.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Y,Z,Z1_coeffs,Z2_coeffs,Intercepts] = generateSystemsData(G,...
    theta,betaReg,...
    alpha_gamma,beta_gamma,mu_gamma,sigma_gamma,dist_gamma,...
    mu_u,sigma_u,mu_z,sigma_z,normalZ)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Generate instruments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if normalZ == 0
    % Assume Z1 and Z2 are independent standard Cauchy
    Z1 = generateUnivariateData(G,4,[],[],[],[]);
    Z2 = generateUnivariateData(G,4,[],[],[],[]);

elseif normalZ == 1 || normalZ == 4
    % Assume Z1 and Z2 are independent Normal(mu_z,sigma_z^2)
    Z1 = generateUnivariateData(G,1,[],[],mu_z,sigma_z);
    Z2 = generateUnivariateData(G,1,[],[],mu_z,sigma_z);
    
end

Z = [Z1 Z2];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Generate additive unobservables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assume U1 and U2 are independent Normal(mu_u,sigma_u^2)
U1 = generateUnivariateData(G,1,[],[],mu_u,sigma_u);
U2 = generateUnivariateData(G,1,[],[],mu_u,sigma_u);
U = [U1 U2];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Generate endogenous variable random coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Independent truncated normal(mu_gamma,sigma_gamm) 
% Note that mu_gamma is not exactly equal to the true mean
if dist_gamma == 1
    gammaVec(:,1) = generateUnivariateData(G,2,[],[],mu_gamma,sigma_gamma);
    gammaVec(:,2) = generateUnivariateData(G,2,[],[],mu_gamma,sigma_gamma);

% Independent Beta(alpha_gamma,rho_gamma)
elseif dist_gamma == 2
    gammaVec(:,1) = generateUnivariateData(G,3,alpha_gamma,...
        beta_gamma,[],[]);
    
    gammaVec(:,2) = generateUnivariateData(G,3,alpha_gamma,...
        beta_gamma,[],[]);

% Independent mixture of normals
elseif dist_gamma == 3 
    gammaVec(:,1) = generateUnivariateData(G,5,alpha_gamma,...
        beta_gamma,mu_gamma,sigma_gamma);
    
    gammaVec(:,2) = generateUnivariateData(G,5,alpha_gamma,...
        beta_gamma,mu_gamma,sigma_gamma);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Generate outcomes by computing the reduced form
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Y = zeros(G,2);
for g=1:G

    Gamma_matrix = [0 gammaVec(g,1);
                    gammaVec(g,2) 0];
 
    Gamma_tilde = (eye(2)-theta*Gamma_matrix);
        
    Y(g,:) = (Gamma_tilde\(betaReg(1) * Z(g,:)' + U(g,:)' ))';
    % For each g, this is a two equation system.
    %
    % U(g,:)' is the column vector of additive unobservables U1 and U2
    %
    % Z_1(g,:)' is a column vector of the values of the instruments
    %
    % betaReg(1) are the coefficients on the instruments in each equation
    %
    % Gamma_tilde is the matrix of random coefficients on the endogenous
    % variables in the structural system.
    %
    % Y(g,:)' is the column vector of outcomes
    
    % Save the reduced form coefficients, for comparison
    Pi = inv(Gamma_tilde)*betaReg(1);
    % This is the matrix of reduced form coefficients
    %  pi_{11}, pi_{12}
    %  pi_{21}, pi_{22}
    
    Z1_coeffs(g,:) = Pi(:,1); % = (pi_{11},pi_{12})
    Z2_coeffs(g,:) = Pi(:,2); % = (pi_{21},pi_{22})
    
    Intercepts(g,:) = Gamma_tilde\U(g,:)'; % = (pi_{13},pi_{23})
    
end

end