%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generateUnivariateData.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: 
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Generates draws from scalar random variables.
%
% dist =
% 1. N(mu,sigma^2)
% 2. Truncated normal
% 3. Beta(alpha,beta)
% 4. Standard Cauchy
% 5. Mixture of normals 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function data = generateUnivariateData(n,dist,alpha,beta,mu,sigma)

% N(mu,sigma^2)
if dist == 1
    
    if sigma <= 0
        error('Sigma must be positive')
    end
    data = mu + sigma*randn(n,1);

% TruncatedGaussian(sigma,[a,b]) has support [a,b] and variance sigma^2
% So here mu is shifting the location of the truncated Gaussian, but it is
% not actually the true mean.
elseif dist == 2
    
    data = mu + TruncatedGaussian(sigma,[0 1]-mu,[n 1]);

% Beta(alpha,beta)
elseif dist == 3
    
    if alpha <= 0 || beta <= 0
        error('Alpha and beta must be positive.')
    end
    data = betarnd(alpha,beta,[n 1]);

% Standard Cauchy
elseif dist == 4
    
    temp = rand(n,1); % N-by-1
    data = tan(pi*(temp-0.5));
    % The standard Cauchy cdf is 
    %   F(t) = (1/pi)*arctan(t)+(1/2)
    % so here we are just converting uniform random draws to have a
    % Cauchy distribution
 
% Mixture of normals    
elseif dist == 5
% This part doesn't use beta
% Also, mu and sigma are 2-vectors
    
    if sigma(1) <= 0 || sigma(2) <= 0
        error('Sigma must be positive')
    end

    temp = rand(n,1);
    index1 = temp < alpha;
    index2 = temp > alpha; %(beta should be 1-alpha)
    
    data1 = mu(1) + sigma(1)*randn(n,1);
    data2 = mu(2) + sigma(2)*randn(n,1);
    
    data = index1.*data1 + index2.*data2;
    
    clear index1 index2 data1 data2

end

end