%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% kernelG.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Nov 7, 2014
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Computes the value of
%
%   G[{lambda}^{-2}.(1-s'e)]
%
% for a vector of values s and e, where
%
% G(t) = I{[0,\infty)}(t)*exp(-t).
%
% This value is used in computing the spherical kernel density estimator,
% which in turn is used in the denominator of the HKM2010 estimator.
%
% INPUTS: 
%   . s,e = points at which the kernel should be evaluated. Here s,e can be
%   points of the 2-dimensional unit circle OR vectors of such points. Thus
%   their dimensions should be 1-by-2 or n-by-2. n can be different for s
%   and e;
%   . lambda = bandwidth.
%
% OUTPUT: 
%   . KG = value taken by the kernel at each pair of elements of the unit 
%   circle that can be obtained combining elements of n and e, so 
%   dim(K) = [ns,ne].
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function KG = kernelG(s,e,lambda)

% s and e are both (vectors of) elements of the d-dimensional unit sphere,
% say they have dimension n-by-d:
% here, d=2.

[ns,d] = size(s);
[ne,d] = size(e);

% Compute the scalar product s'e
smat1 = s(:,1)*ones(1,ne); % ns-by-ne
smat2 = s(:,2)*ones(1,ne); % ns-by-ne
emat1 = ones(ns,1)*e(:,1)'; % ns-by-ne
emat2 = ones(ns,1)*e(:,2)'; % ns-by-ne
% Same as:
% se = s(:,1)*e(:,1)' + s(:,2)*e(:,2)';

se = emat1.*smat1 + emat2.*smat2;
clear smat1 smat2 emat1 emat2

% Argument for the G function
input = (1-se)/(lambda^2);
clear se

indic = (input > 0);
KG = indic.*exp(-input);

end