%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% kernelK.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Nov 11, 2014
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% DESCRIPTION: Implements the kernel K_{r,h} involved in the NUMERATOR step
% of the HKM2010 estimator.
%
% K_{r,h}(a)
%  = (1/(2*pi)^d) \int_{0}^{1/h} cos(at)*t^{d-1} dt   IF r = \infty
%  = (1/(2*pi)^d) \int_{0}^{1/h} cos(at)*t^{d-1}[1-(ht)^r] dt   IF
%    0 < r < \infty
%
% INPUTS: 
%   . a = points at which the kernel should be evaluated, can be a matrix
%   of a vector;
%   . Tgrid = grid for t, over which the integration (involved in the
%   inverse Fourier transform defining Khr) is performed;
%   . Tstep = step size of Tgrid, needed for the numerical integration;
%   . dimension d, order r = parameters as specified in the paper (p.810);
%   . h = bandwidth.
%
% OUTPUT: 
%   . Khr = value taken by the kernel at each point of the input vector or
%   matrix a - so dim(Khr) = dim(a)
%
% NOTES:
%
% Given that (1) fhat = num./denom should be non-negative everywhere, and 
% (2) denom is positive everywhere; num should be also non-negative
% everywhere. 
% sin is negative when angle \in [-pi,0] or [pi,2pi]
% cos is negative when angle \in [-pi,-pi/2]U[pi/2,pi] or [pi/2,3pi/2] 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Khr = kernelK(a,d,r,h,Tgrid,Tstep) 

%a = SbU;
if r==1 && d==2
    caserd = 1;
    
elseif r==1 && d==3
    caserd = 2;
    
elseif r==2 && d==3
    caserd = 4;
    
else
    caserd = 3;
    
end

switch caserd
    case 1
        %r=1, d=2
        K = ((2*h*sin(a/h)-a-a.*cos(a/h))./(a.^3))/(4*pi^2);
        
        bound0 = (1./(4*(pi^2)*(h^2)*6));
        indic0 = a==0;
        if sum(indic0)>0
            index0 = find(a==0);
            K(index0) = bound0;
            clear index0 indic0
        end
        
        Khr = K;
        clear K
        
    case 2 
        %r=1, d=3
        K = ((4*a*h.*sin(a/h)-6*h^2-(a.^2-6*h^2).*cos(a/h))./(h*a.^4))/(8*pi^3);
        
        bound0 = (1./(8*(pi^3)*(h^3)*12));
        indic0 = a==0;
        
        if sum(sum(indic0))>0
            index0 = find(a==0);
            K(index0) = bound0;
            clear index0 indic0
        end
        
        indicbound = abs(K) > bound0;
        if sum(sum(indicbound))>0
            indexbound = find(abs(K) > bound0);
            K(indexbound) = bound0;
            clear indexbound indicbound
        end
        
        Khr = K;
        clear K
        
    case 4
        % r=2, d=3
        K = (-2*a.*(a.^2 - 12*h^2).*cos(a/h) + 2*h*(5*a.^2 - 12*h^2).*sin(a/h))./((a.^5)*h)/((2*pi)^3);
        
        bound0 = (2./(8*(pi^3)*(h^3)*15));
        indic0 = a==0;
        
        if sum(sum(indic0))>0
            index0 = find(a==0);
            K(index0) = bound0;
            clear index0 indic0
        end
        
        indicbound = abs(K) > bound0;
        if sum(sum(indicbound))>0
            indexbound = find(abs(K) > bound0);
            K(indexbound) = bound0;
            clear indexbound indicbound
        end
        
        Khr = K;
        clear K
        
        
    case 3
        mat = 1;
        if length(a(1,:))>1
            mat = 1;
        end

        %% if a is a (column) vector
        if mat==0
            % a should be a column: Na-by-1
            Na = length(a);
            % Tgrid should be a row: 1-by-Nt
            Nt = length(Tgrid);
            cosat = cos(a*Tgrid); % Na-by-Nt; 

            % t^{d-1};
            %d=2;
            td = Tgrid.^(d-1); % 1-by-Nt
            indic = ((h*Tgrid)>0).*((h*Tgrid)<1); % 1-by-Nt
            td = td.*indic; % 1-by-Nt

            % L_{r}(h|t|) = [1-(ht)^r]
            if r < Inf
                htr = 1-(h*Tgrid).^r; % 1-by-Nt
                td = td.*htr; % 1-by-Nt
            end

            td = ones(Na,1)*td; % Na-by-Nt
            integrand = cosat.*td; % Na-by-1

            % Numerical integration over t: sum over the columns
            Khr = sum(integrand*Tstep,2); % Na-by-1
            Khr = Khr/((2*pi)^d);

        %% if a is a matrix
        else
            % get dimensions
            [Na,Nb] = size(a);
            Nt = length(Tgrid);
            % Now the goal is to compute at, in which each of the Na*Nb elements of 
            % a should be multplied by each of the Nt elements of Tgrid. So we want
            % at to be Na-by-Nb-by-Nt
            % (and we don't want to use the function 'repmat' because it gets too
            % slow).

            % t^{d-1};
            % d=2;
            td = Tgrid.^(d-1); % 1-by-1-by-Nt
            indic = ((h*Tgrid)>0).*((h*Tgrid)<1); % 1-by-1-by-Nt
            td = td.*indic; % 1-by-1-by-Nt

            % L_{r}(h|t|) = [1-(ht)^r]
            if r < Inf
                htr = 1-(h*Tgrid).^r; % 1-by-1-by-Nt
                td = td.*htr; % 1-by-1-by-Nt
            end

            % why do I loop over grid points? See below, at the end for the past
            % version without loop, that was failing
            Khr = zeros(Na,Nb);
            for ti=1:Nt
                t = Tgrid(ti); % scalar
                temp = cos(a*t)*td(ti); % Na-by-Nb
                Khr = Khr+temp*Tstep; % Na-by-Nb
            end
            Khr = Khr/((2*pi)^d); % Na-by-Nb
        end

end

end

%% Figure 3 from the paper HKM2010 (p.811) to check the function
%
% d=2; 
% r=1; 
% hvec=[0.3,0.5,0.6];
% a = [0.05:0.1:15]'; % a should be a column: Na-by-1
% Tstep = 0.02;
% Tgrid = 0:Tstep:100; % Tgrid should be a row: 1-by-Nt
% 
%  for hi=1:3
%      h = hvec(hi);
%      %Khr(:,hi) = kernelK(a,Tgrid,Tstep,d,r,h);
%  end
% 
%     plot(a,Khr(:,1))
%     hold on
%     plot(a,Khr(:,2),'r')
%     plot(a,Khr(:,3),'g')
%     hold off
%     title(sprintf('Test kernel Khr function, d=%d, r=%d',...
%                         d,r));
%     legend(gca,'h=0.3','h=0.5','h=0.6');
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% WHY DO I LOOP?
%
% My initial strategy was to use a 3-dimensional array, but it is too big
% for Matlab to handle it; so returns `out-of-memory'
%
%     Tgrid = reshape(Tgrid,1,1,Nt); % 1-by-1-by-Nt
%     temp = bsxfun(@times,ones(Na,Nb,Nt),Tgrid); % Na-by-Nb-by-Nt
%     at = bsxfun(@times,a,temp); % Na-by-Nb-by-Nt; 
%     cosat = cos(at); % Na-by-Nb-by-Nt; 
%     clear temp at
% 
%     % t^{d-1};
%     % d=2;
%     td = Tgrid.^(d-1); % 1-by-1-by-Nt
%     indic = ((h*Tgrid)>0).*((h*Tgrid)<1); % 1-by-1-by-Nt
%     td = td.*indic; % 1-by-1-by-Nt
% 
%     % L_{r}(h|t|) = [1-(ht)^r]
%     if r < Inf
%        htr = 1-(h*Tgrid).^r; % 1-by-1-by-Nt
%        td = td.*htr; % 1-by-1-by-Nt
%     end
%
%     temp = bsxfun(@times,ones(Na,Nb,Nt),td); % Na-by-Nb-by-Nt
%     integrand = cosat.*temp; % Na-by-Nb-Nt
% 
%     % Numerical integration over t: sum over the columns
%     Khr = sum(integrand*Tstep,3); % Na-by-Nb
%     Khr = Khr/((2*pi)^d);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
