%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% printMCresults.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Sept 27, 2013
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function printMCresults(tableNumber)
%% Load results from each dgp/sample size pair

[dgpGrid, nSimulations, ~] = setnSimsAndDgpGrid();

if tableNumber == 1
% Cauchy Z, main dgp 

    % Column 1 = sampleSize
    % Column 2 = dgpChoice
    % Column 3 = nSimulations
    % Column 4 = normalZ
    dgps = [500 1 nSimulations 0; ...   % Indep truncated normal(0.4,0.5)
            1000 1 nSimulations 0; ...
            500 3 nSimulations 0; ...   % Indep Beta(6,3)
            1000 3 nSimulations 0; ...
            500 5 nSimulations 0; ...
            1000 5 nSimulations 0];

elseif tableNumber == 2
% Normal Z, main dgp

    dgps = [500 1 nSimulations 1; ...   % Indep truncated normal(0.4,0.5)
            1000 1 nSimulations 1; ...
            500 3 nSimulations 1; ...   % Indep Beta(6,3)
            1000 3 nSimulations 1; ...
            500 5 nSimulations 1; ...
            1000 5 nSimulations 1; ...
            500 7 nSimulations 3]; % Empirical dgp
        
end

dgp_names = {'Truncated normal(0.4,0.05)','',...
    'Beta(6,3)','','Bimodal','','Empirical Bimodal'};

dgp_trueMeans = {'$\Exp(\gamma) = 0.38$','',...
    '$\Exp(\gamma) = 0.63$','','$\Exp(\gamma) = 0.551$',...
    '','$\Exp(\gamma) = 0.551$'};

dgp_trueMedian = {'$\text{Med}(\gamma) = 0.38$','',...
    '$\text{Med}(\gamma) = 0.6455$','','$\text{Med}(\gamma) = 0.6327$',...
    '','$\text{Med}(\gamma) = 0.6327$'};

dgp_trueIQR = {'$\text{IQR} = 0.0641$','',...
    '$\text{IQR} = 0.202$','','$\text{IQR} = 0.355$',...
    '','$\text{IQR} = 0.355$'};

% Load results
for i=1:size(dgps,1)
    
    sampleSize = dgps(i,1);
    dgpChoice = dgps(i,2);
    normalZ = dgps(i,4);

    load(sprintf('../output_MCresults/MCresults_n%d_dgp%d_nZ%d.mat',...
        sampleSize,dgpChoice,normalZ));

    % Row 1
    pointEsts(i,:) = [MCresults.mean_gamma_medbias_TSLS ...
                      MCresults.mean_gamma_medbias_nonpara ...
                      MCresults.median_gamma_medbias_nonpara ...
                      MCresults.IQR_gamma_medbias_nonpara ...                     
                      MCresults.median_ISE];
    % Row 2
    IQRs(i,:) = [MCresults.mean_gamma_TSLS_IQR ...
                           MCresults.mean_gamma_nonpara_IQR ...
                           MCresults.median_gamma_nonpara_IQR ...
                           MCresults.IQR_gamma_nonpara_IQR ...
                           MCresults.MISE_IQR];
             
end


%% Output Mean results to Latex table

% 6 rows, for each dgp and sample size pair.
% 5 columns, for each statistic of interest
fid = fopen(sprintf('../output_MCresults/MCsim_results_table%d.tex',tableNumber),'w');

fprintf(fid, '\\begin{table} \\centering');

if tableNumber == 1
fprintf(fid, '\\caption{Monte Carlo results: Cauchy $Z$ \\label{MCresultsTable_median}} \n');
elseif tableNumber == 2
fprintf(fid, '\\caption{Monte Carlo results: Normal $Z$ \\label{MCresultsTable_median_normalZ}} \n');  
end

fprintf(fid, '\\setlength{\\linewidth}{.1cm} \n');
fprintf(fid, '\\newcommand{\\contents}{ \n');

fprintf(fid, '\\begin{tabular}{l c c c c c} \\hline\\hline \n');
fprintf(fid, '& \\multicolumn{2}{c}{$\\widehat{\\Exp}(\\gamma)$} & $\\widehat{\\text{Med}}(\\gamma)$ & $\\widehat{\\text{IQR}}(\\gamma)$ & Median ISE \\\\ \n');
fprintf(fid, '& 2SLS & RC & RC & RC & RC \\\\ \\hline \n');
for i=1:size(dgps,1)
    fprintf(fid, '%s & \\multicolumn{2}{c}{%s} & %s & %s & \\\\ \n', ...
        dgp_names{i},dgp_trueMeans{i},dgp_trueMedian{i},dgp_trueIQR{i});
    fprintf(fid, '$N=%8.0f$ & %8.4f & %8.4f  & %8.4f  & %8.4f & %8.4f \\\\ \n ', ...
        dgps(i,1),pointEsts(i,1),pointEsts(i,2),pointEsts(i,3),pointEsts(i,4),pointEsts(i,5));
    fprintf(fid, '& $[%8.4f]$ & $[%8.4f]$  & $[%8.4f]$  & $[%8.4f]$ & $[%8.4f]$ \\\\ \n', ...
        IQRs(i,1),IQRs(i,2),IQRs(i,3),IQRs(i,4),IQRs(i,5));
    
    if mod(i,2) == 0
        fprintf(fid, '\\\\');
    end
    
    if i==size(dgps,1)
        fprintf(fid, '\\hline\\hline ');
    end    
    
    fprintf(fid, '\n');
end
fprintf(fid, '\\multicolumn{6}{p{\\linewidth}}{\\footnotesize For each dgp: Median bias is first. IQR in brackets.} \\\\ \n');

fprintf(fid, '\\end{tabular}\n');

fprintf(fid, '} \n');

fprintf(fid, '\\setbox0=\\hbox{\\contents} \n');
fprintf(fid, '\\setlength{\\linewidth}{\\wd0-2\\tabcolsep-.25em} \n');
fprintf(fid, '\\contents \n');

fprintf(fid, '\\end{table}\n');
fclose(fid);


end