%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% runEmpiricalAnalysis.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: May 17, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function runEmpiricalAnalysis(M,masterBandwidth)

rng(28591,'twister');

%% Set parameters

B = 250; % Number of bootstrap draws
nQuantileGrid = 1000;
alpha = 0.05; % For bootstrap confidence intervals

bootstrapOn = 0;

negativeLowerBound = 1;
% 0 = original estimate in cemmap 2015 working paper version
% 1 = final paper version

%{
M = 10; % This is the baseline tuning parameter
masterBandwidth = 0.1;
%}

%% Load data

data = csv2mat_numeric('/Volumes/untitled/data/datasets/fullSample.csv');

Y = [data.currentGPA_1 data.currentGPA_2];
Z = [data.laggedGPA_1 data.laggedGPA_2];

clear data;

sampleSize = size(Y,1);

%% Compute main empirical estimate
EmpiricalOutput1 = runWithEmpiricalData(Y,Z,negativeLowerBound,M,masterBandwidth);
EmpiricalOutput2 = runWithEmpiricalData([Y(:,2) Y(:,1)],[Z(:,2) Z(:,1)],negativeLowerBound,M,masterBandwidth);

fgamma2HatRescaled1 = EmpiricalOutput1.fgamma2HatRescaled;
fgamma2HatRescaled2 = EmpiricalOutput2.fgamma2HatRescaled;

Zstep = EmpiricalOutput1.Zstep;
Zgrid = EmpiricalOutput1.Zgrid;
DgpParams = EmpiricalOutput1.DgpParams;

fgammaHat = (fgamma2HatRescaled1 + fgamma2HatRescaled2)/2;

%% Compute bootstrap estimates
if bootstrapOn == 1

fgammaHatb = nan(length(Zgrid),B);
fgammaHatMeanb = nan(B,1);
fgammaHatMedianb = nan(B,1);
quantile75b = nan(B,1);
quantile25b = nan(B,1);
fgammaHatIQRb = nan(B,1);
probPostiveb = nan(B,1);

for b = 1:B
tic
    display(sprintf('Bootstrap draw = %g',b));

    % Draw bootstrap dataset
    databoot = datasample([Y Z], sampleSize);
    Yb = [databoot(:,1) databoot(:,2)];
    Zb = [databoot(:,3) databoot(:,4)];

    mean(Yb)
    mean(Zb)
        
    % Estimate density
    EmpiricalOutput1b = runWithEmpiricalData(Yb,Zb,negativeLowerBound,M,masterBandwidth);
    EmpiricalOutput2b = runWithEmpiricalData([Yb(:,2) Yb(:,1)],[Zb(:,2) Zb(:,1)],negativeLowerBound,M,masterBandwidth);

    fgamma2HatRescaled1b = EmpiricalOutput1b.fgamma2HatRescaled;
    fgamma2HatRescaled2b = EmpiricalOutput2b.fgamma2HatRescaled;

    % Save estimate
    fgammaHatb(:,b) = (fgamma2HatRescaled1b + fgamma2HatRescaled2b)/2;
    
    % Compute summary stats
    fgammaHatMeanb(b) = sum(Zgrid.*fgammaHatb(:,b))*Zstep;
    
    fgammaHatMedianb(b) = computeQuantile(nQuantileGrid,Zgrid,fgammaHatb(:,b),0.5,...
        DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

    quantile75b(b) = computeQuantile(nQuantileGrid,Zgrid,fgammaHatb(:,b),0.75,...
        DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

    quantile25b(b) = computeQuantile(nQuantileGrid,Zgrid,fgammaHatb(:,b),0.25,...
        DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

    fgammaHatIQRb(b) = quantile75b(b) - quantile25b(b);
    
    probPostiveb(b) = 1-computeCdf(nQuantileGrid,Zgrid,fgammaHatb(:,b),0,...
        DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);
    
    save(sprintf('../empiricalOutput/empiricalEstimates_%d_%d_%2.4f.mat',negativeLowerBound,M,masterBandwidth));
toc
end

%% Get bootstrap percentile method confidence intervals

fgammaHatMean_LB = quantile(fgammaHatMeanb, alpha/2, 1);
fgammaHatMedian_LB = quantile(fgammaHatMedianb, alpha/2, 1);
quantile75_LB = quantile(quantile75b, alpha/2, 1);
quantile25_LB = quantile(quantile25b, alpha/2, 1);
fgammaHatIQR_LB = quantile(fgammaHatIQRb, alpha/2, 1);
probPostiveb_LB = quantile(probPostiveb, alpha/2, 1);

fgammaHatMean_UB = quantile(fgammaHatMeanb, 1- alpha/2, 1);
fgammaHatMedian_UB = quantile(fgammaHatMedianb, 1- alpha/2, 1);
quantile75_UB = quantile(quantile75b, 1- alpha/2, 1);
quantile25_UB = quantile(quantile25b, 1- alpha/2, 1);
fgammaHatIQR_UB = quantile(fgammaHatIQRb, 1- alpha/2, 1);
probPostiveb_UB = quantile(probPostiveb, 1- alpha/2, 1);

%% Get bootstrap confidence band
fgammaHat_upperBand = nan(size(fgammaHatb,1),1);
fgammaHat_lowerBand = nan(size(fgammaHatb,1),1);

for vi = 1:size(fgammaHatb,1);
    
    fgammaHat_upperBand(vi) = ...
        quantile(fgammaHatb(vi,:),0.975);
    
    fgammaHat_lowerBand(vi) = ...
        quantile(fgammaHatb(vi,:),0.025);
    
end

end

%% Plot main estimate

printDirectory = sprintf('../empiricalOutput/fgammaHatempirical_%d_%d_%2.4f.eps',negativeLowerBound,M,masterBandwidth);

if bootstrapOn == 0    
    fgammaHat_upperBand = zeros(size(fgammaHat,1),1);
    fgammaHat_lowerBand = zeros(size(fgammaHat,1),1);
end

plotFinalPdfEstimateEmpirical(DgpParams,fgammaHat,Zgrid,...
    printDirectory,fgammaHat_upperBand,fgammaHat_lowerBand);

%% Compute estimated summary stats for the main estimate

fgammaHatMean = sum(Zgrid.*fgammaHat)*Zstep;

fgammaHatMedian = computeQuantile(nQuantileGrid,Zgrid,fgammaHat,0.5,...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

quantile75 = computeQuantile(nQuantileGrid,Zgrid,fgammaHat,0.75,...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

quantile25 = computeQuantile(nQuantileGrid,Zgrid,fgammaHat,0.25,...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

fgammaHatIQR = quantile75 - quantile25;

probPostive = 1-computeCdf(nQuantileGrid,Zgrid,fgammaHat,0,...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

%% Find proportion in the two modes (as reported in the paper body text):
prop1 = computeCdf(nQuantileGrid,Zgrid,fgammaHat,0,...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

prop2 = computeCdf(nQuantileGrid,Zgrid,fgammaHat,0.4,...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

prop3 = computeCdf(nQuantileGrid,Zgrid,fgammaHat,0.55,...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

prop4 = computeCdf(nQuantileGrid,Zgrid,fgammaHat,0.8,...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);

mode1mass = prop2 - prop1;
mode2mass = prop4 - prop3;

display([mode1mass mode2mass]);

%% display result
if bootstrapOn == 1
    
display([fgammaHatMean_LB fgammaHatMean fgammaHatMean_UB]);
display([quantile25_LB quantile25 quantile25_UB]);
display([fgammaHatMedian_LB fgammaHatMedian fgammaHatMedian_UB]);
display([quantile75_LB quantile75 quantile75_UB]);
display([fgammaHatIQR_LB fgammaHatIQR fgammaHatIQR_UB]);
display([probPostiveb_LB probPostive probPostiveb_UB]);

end

%% Supplemental online appendix section B.4:
% Compute proportion of observations with out of support predicted Y's
% (Commented out because it doesn't always need to be computed)

%{

nSims = 100;

betaEst = 0.78;
muEst = 0;

for s=1:nSims
       
% gammaMax = 0.15;

% want to randomly sample from the estimated density of gamma's
% then assign those to each person. Get P(GPApred not in [0,4]) averaged
% over lots of samples

uniformDraw1 = rand(sampleSize,1);
uniformDraw2 = rand(sampleSize,1);

gammaVec1 = nan(sampleSize,1);
gammaVec2 = nan(sampleSize,1);

for i=1:sampleSize
    
gammaVec1(i) = computeQuantile(nQuantileGrid,Zgrid,fgammaHat,uniformDraw1(i),...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);    
    
gammaVec2(i) = computeQuantile(nQuantileGrid,Zgrid,fgammaHat,uniformDraw2(i),...
    DgpParams.gammaLowerBound,DgpParams.gammaUpperBound);    
    
end

gpaPredicted1 = muEst + gammaVec1.*Z(:,1) + betaEst*Z(:,2);
gpaPredicted2 = muEst + gammaVec2.*Z(:,2) + betaEst*Z(:,1);

gpaPredicted = [gpaPredicted1;
                gpaPredicted2];
            
outOfBounds = (gpaPredicted > 4) + (gpaPredicted < 0);
mean1(s) = mean(outOfBounds);

outOfBounds2 = (gpaPredicted > 5) + (gpaPredicted < 0);
mean2(s) = mean(outOfBounds2);

end

mean(mean1)
mean(mean2)

%}

%% Save output

save(sprintf('../empiricalOutput/empiricalEstimates_%d_%d_%2.4f.mat',negativeLowerBound,M,masterBandwidth));

end
