%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% runWithEmpiricalData.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: May 11, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
% NOTE: 
% negativeLowerBound is used by setDgpParamsEmpirical.m
% M is used by setGridParametersEmpirical.m
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function EmpiricalOutput = runWithEmpiricalData(Y,Z,negativeLowerBound,M,masterBandwidth)

sampleSize = size(Y,1);

%% Set parameters
%% IMPORTANT: This code is setup to pick the bandwidth to minimize the ISE
% from the dgpChoice density. So that's the main reason this choice will
% affect the resulting dgp estimate. Moreover, these parameters all affect
% the choice of the numerical integration grids

% Choose the distribution of gamma
dgpChoice = 4;

% Define dgp relevant parameters
run setDgpParamsEmpirical.m;

% Define numerical approximation parameters
run setGridParametersEmpirical.m;
% Requires the parameter M to be defined already

GridOutputs = computeTandSgridsEmpirical(GridParams,DgpParams);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Estimation specific stuff starts here
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Estimation parameters

% Bandwidths for Radon estimator
EstimationParams.radonBandwidths = 0.6;

EstimationParams.masterBandwidth = masterBandwidth;

EstimationParams.masterBandwidthBaselineRange = 1;

EstimationParams.r = 1; % Parameter for inverse Radon transform Kernel
EstimationParams.lambda = 0.2; % Spherical kernel bandwidth;
EstimationParams.tau = 0; % Amount of trimming for denominator density

% Compute spherical data kernel estimator normalization constant
EstimationParams.clambda = ...
    computeSphericalKernelConstant(EstimationParams.lambda);

%% Prep data

X = [Z ones(sampleSize,1)]; %sampleSize-by-3
% Y is loaded here too, sampleSize-by-2

% Compute linear combinations of the outcome variables
% We will pass these to the estimator

YlinComb = Y * (GridOutputs.Tgrid'); % sampleSize-by-nTgrid
% Y is sampleSize-by-2
% GridOutputs.Tgrid is nTgrid-by-2
% so GridOutputs.Tgrid' is 2-by-nTgrid
%
% Y = ( Y1 Y2 )
%     (  .  . )
%        .
%        .
%
% GridOutputs.Tgrid = (t1 t2)
%                       .  .
%                       .  .
%
% Y * (GridOutputs.Tgrid') = (Y1 Y2) ( t1 . .
%                              .  .  ( t2 . .
%
% So Y(:,it) is a sampleSize-by-1 vector of
%       t1*Y1 + t2*Y2
% where (t1,t2) are the it'th row of Tgrid

%% Estimate

[linCombPdfHat, linCombPdfHatRescaled, ...
    optimalBandwidth, optimalBandwidthRescaled] = ...
    computeEstimatedFirstStageEmpirical(GridParams,GridOutputs,...
        EstimationParams,[],X,YlinComb);
    
linCombPdfHat = linCombPdfHat'; % nSgrid-by-nTgrid
linCombPdfHatRescaled = linCombPdfHatRescaled'; % nSgrid-by-nTgrid

[fgamma2Hat, Zgrid] = ...
    computeSecondStage(GridParams,DgpParams,GridOutputs,linCombPdfHat);

% Truncate and rescale our estimated densities
Zstep = range(Zgrid)/(GridParams.nZgrid-1);
% This step size is used here and several places below

fgamma2HatTruncated = max(fgamma2Hat,0);
integral = sum(fgamma2HatTruncated)*Zstep;
fgamma2HatRescaled = fgamma2HatTruncated/integral;

%% Save

EmpiricalOutput.fgamma2HatRescaled = fgamma2HatRescaled;
EmpiricalOutput.Zstep = Zstep;
EmpiricalOutput.Zgrid = Zgrid;
EmpiricalOutput.DgpParams = DgpParams;

end

