%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% runWithEstimatedFirstStage.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: March 21, 2016
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function runWithEstimatedFirstStage(...
    sampleSize,GridParams,DgpParams,dgpChoice,EstimationParams,...
    normalZ,simulationNumber,GridOutputs,useCluster,dgpGridChoice)
%% For picking which first stage to use
useRescaledBandwidths = 1;

%% For cluster in parallels

if useCluster == 1 % Running on cluster
    % Get the current job_id from the server.
    job_id_string = getenv('SLURM_ARRAY_TASK_ID');
    job_id = str2double(job_id_string);
    
    % When using the cluster, the input simulationNumber gets ignored

elseif useCluster == 0 % Run locally

    job_id = simulationNumber;

end

%% Load data

   load(sprintf('../datasets/MCdataset_n%d_dgp%d_nZ%d_sim%d.mat',...
      sampleSize,dgpChoice,normalZ,job_id));
       
    X = [Z ones(sampleSize,1)]; %sampleSize-by-3
        % Y is loaded here too, sampleSize-by-2
        % Compute linear combinations of the outcome variables
        % We will pass these to the estimator
        
    YlinComb = Y * (GridOutputs.Tgrid'); % sampleSize-by-nTgrid
        % Y is sampleSize-by-2
        % GridOutputs.Tgrid is nTgrid-by-2
        % so GridOutputs.Tgrid' is 2-by-nTgrid
        %
        % Y = ( Y1 Y2 )
        %     (  .  . )
        %        .
        %        .
        %
        % GridOutputs.Tgrid = (t1 t2)
        %                       .  .
        %                       .  .
        %
        % Y * (GridOutputs.Tgrid') = (Y1 Y2) ( t1 . .
        %                              .  .  ( t2 . .
        %
        % So Y(:,it) is a sampleSize-by-1 vector of
        %       t1*Y1 + t2*Y2
        % where (t1,t2) are the it'th row of Tgrid


%% Estimate
    display('Estimating...');
    computingMISEs = 0;

    [linCombPdfHat, linCombPdfHatRescaled] = ...
        computeEstimatedFirstStage(...
        GridParams,GridOutputs,EstimationParams,X,YlinComb,computingMISEs,dgpGridChoice);
 
    linCombPdfHat = linCombPdfHat'; % nSgrid-by-nTgrid
    linCombPdfHatRescaled = linCombPdfHatRescaled'; % nSgrid-by-nTgrid

%%

    display('Second stage...');
    
    if useRescaledBandwidths == 0
    
        [fgamma2Hat, Zgrid] = ...
            computeSecondStage(GridParams,DgpParams,...
                GridOutputs,linCombPdfHat);

    elseif useRescaledBandwidths == 1
        
        [fgamma2Hat, Zgrid] = ...
            computeSecondStage(GridParams,DgpParams,...
                GridOutputs,linCombPdfHatRescaled);
    
    end
    
    % True pdf
    fgamma2True = computeUnivariatePdf(Zgrid,DgpParams); % nZgrid-by-1
    
    if useCluster == 0
        plotFinalPdfEstimateVsTruth(GridParams,DgpParams,...
            fgamma2True,fgamma2Hat,Zgrid,dgpChoice);
    end
    
    save(sprintf('../output_secondStage/MCestimates_n%d_dgp%d_nZ%d_sim%d.mat',...
        sampleSize,dgpChoice,normalZ,job_id),...
        'Zgrid','GridParams','DgpParams','fgamma2Hat',...
        'fgamma2True','linCombPdfHat', '-v7.3');

end
