%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% setDgpParams.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: March 29, 2016
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function DgpParams = setDgpParams(dgpChoice)

DgpParams.dgpChoice = dgpChoice;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% True coefficient on the instruments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Note: beta > 0 must hold for the truePi.m code to be correct. We also 
% need 1 - gamma1 * gamma2 > 0 to always hold. This is *only* for the 
% truePi.m code to be correct. Not for the second step to work using an 
% estimated first step.

if dgpChoice == 7 % Empirically calibrated dgp
    DgpParams.betaZcoeff = 1;
else
    DgpParams.betaZcoeff = 5;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% Intercept/location of additive unobservables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if dgpChoice == 7
    DgpParams.intercept = -2;
else
    DgpParams.intercept = -10;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% Define parameters for the distributions of gamma1 and gamma2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if dgpChoice == 1

    DgpParams.dist = 1; % 1 = Independent truncated Normal(mu,sigma^2)
    
    DgpParams.mu = 0.4; % Note: this is the mean BEFORE truncation. It does not
                        % necessarily equal the mean after truncation.
                        
    DgpParams.sigma = 0.05;

    % Support of the distribution (i.e., the truncation points)
    DgpParams.alpha = 0; 
    DgpParams.beta = 1;
    
elseif dgpChoice == 3
    
    DgpParams.dist = 2; % 2 = Beta(alpha,beta)
    DgpParams.alpha = 6;
    DgpParams.beta = 3; % Note: Mean = alpha/(alpha+beta)

    % Unused parameters
    DgpParams.mu = [];
    DgpParams.sigma = [];
    
elseif dgpChoice == 5 || dgpChoice == 7
        
    DgpParams.dist = 3; % 3 = mixture of alpha*Normal(mu_1,sigma_1^2) and 
                        % beta*Normal(mu_2,sigma_2^2)
    DgpParams.mu = [.3;.7];
    DgpParams.sigma = [.05,.06];
    
    % Mixture weights
    DgpParams.alpha = 0.3;
    DgpParams.beta = 0.7;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Shift and scale the distributions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% We start with one of the distributions supported on [0,1] above. We then
% convert its support to [shift,shift + scale] by 
% scaling it by scale and then shifting it by adding shift.

DgpParams.shift = 0;
DgpParams.scale = 0.95;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Compute support of gamma1 and gamma2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% For various integrals and grids we need to specify the support of the
% random coefficients.

% WARNING: THESE PARAMETERS ALSO AFFECT COMPUTATION OF THE TRUE ANALYTICAL 
% DENSITY OF LINEAR COMBINATIONS

% The support of gamma1 and of gamma2
DgpParams.gammaLowerBound = DgpParams.shift;
DgpParams.gammaUpperBound = DgpParams.shift + DgpParams.scale;

if DgpParams.gammaUpperBound < DgpParams.gammaLowerBound
    error('Support of gamma is incorrect; please check parameters.');
end

if DgpParams.gammaLowerBound < -1 || DgpParams.gammaUpperBound > 1
    error('Gamma support should be between -1 and 1');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Compute support of the reduced form coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The support of the reduced form coefficients is used for defining grids
% over which we integrate pdfs for these coefficients

% These parameters are used for analytical computations involving the true
% pdf of reduced form parameters, in the simulations. In my examples I
% assumed 
%
% (1-gamma1*gamma2) > 0
%
% so I can't allow negative gamma's, when doing these analytical
% computations. Hence I did not code these two cases below. To be clear,
% however, this is irrelevant for the estimator itself. That is, for
% estimation and the simulations I can certainly allow for negative
% gamma's.

% These calculations assume that | gamma_i | < 1 for both i=1,2
if DgpParams.gammaLowerBound >= 0
    %
    % pi_{12} = beta/(1-gamma1*gamma2)
    %    
    DgpParams.pi12lowerBound = ...
        DgpParams.betaZcoeff/(1-DgpParams.gammaLowerBound^2);
    
    DgpParams.pi12upperBound = ...
        DgpParams.betaZcoeff/(1-DgpParams.gammaUpperBound^2);
    
    %
    % pi_{22} = beta*gamma2/(1-gamma1*gamma2)
    %
    DgpParams.pi22lowerBound = ...
        DgpParams.betaZcoeff*DgpParams.gammaLowerBound/...
        (1-DgpParams.gammaLowerBound^2);
    
    DgpParams.pi22upperBound = ...
        DgpParams.betaZcoeff*DgpParams.gammaUpperBound/...
        (1-DgpParams.gammaUpperBound^2);
    
    %
    % pi_{11} = (U1 + gamma1*U2)/(1-gamma1*gamma2)
    %
    
    UsupportLowerBound = -2;
    UsupportUpperBound = 2;
    
    % Here I'm assuming that the support of U1 and U2 is
    %
    % (-UsupportLowerBound,UsupportUpperBound) + DgpParams.intercept
    %
    % For example, this is essentially the case if they are 
    % independent N(0,1)
    DgpParams.pi11lowerBound = ...
        ((UsupportLowerBound+DgpParams.intercept)...
        +(UsupportLowerBound+DgpParams.intercept)...
        *DgpParams.gammaUpperBound)/...
        (1-DgpParams.gammaUpperBound^2);

    if UsupportUpperBound + DgpParams.intercept < 0
        
        DgpParams.pi11upperBound = ...
            ((UsupportUpperBound+DgpParams.intercept)...
            +(UsupportUpperBound+DgpParams.intercept)...
            *DgpParams.gammaLowerBound)/...
            (1-DgpParams.gammaLowerBound^2);
    
    elseif UsupportUpperBound + DgpParams.intercept >= 0

        DgpParams.pi11upperBound = ...
            ((UsupportUpperBound+DgpParams.intercept)...
            +(UsupportUpperBound+DgpParams.intercept)...
            *DgpParams.gammaUpperBound)/...
            (1-DgpParams.gammaUpperBound^2); 
    
    end
     
    %
    % pi_{21} = (U2 + gamma2*U1)/(1-gamma1*gamma2)
    %
    DgpParams.pi21lowerBound = DgpParams.pi11lowerBound;
    DgpParams.pi21upperBound = DgpParams.pi11upperBound;
    
    %
    % pi_{13} = gamma1*beta/(1-gamma1*gamma2)
    %
    DgpParams.pi13lowerBound = DgpParams.pi22lowerBound;
    DgpParams.pi13upperBound = DgpParams.pi22upperBound;
    
    %
    % pi_{23} = beta/(1-gamma1*gamma2)
    %
    DgpParams.pi23lowerBound = DgpParams.pi12lowerBound;
    DgpParams.pi23upperBound = DgpParams.pi12upperBound;
    
elseif DgpParams.gammaLowerBound < 0 && DgpParams.gammaUpperBound > 0
 
    % This case is not necessary
    error('Support of gamma computations incomplete');
    
elseif DgpParams.gammaUpperBound <= 0
    
    % This case is not necessary
    error('Support of gamma computations incomplete');
    
end

end