%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% setDgpParamsEmpirical.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: May 14, 2015
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DgpParams.dgpChoice = dgpChoice;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% True coefficient on the instruments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DgpParams.betaZcoeff = 0.7;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% Intercept/location of additive unobservables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DgpParams.intercept = 0.04;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% Define parameters for the distributions of gamma1 and gamma2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if dgpChoice == 4
    
    DgpParams.dist = 2; % 2 = Beta(alpha,beta)

    DgpParams.alpha = 3;
    DgpParams.beta = 3; % Note: Mean = alpha/(alpha+beta)

    %Unused parameters
    DgpParams.mu = [];
    DgpParams.sigma = [];
   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Compute support of gamma1 and gamma2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% We start with one of the distributions supported on [0,1] above. We then
% convert its support to [shift,shift + scale] by 
% scaling it by scale and then shifting it by adding shift.

if negativeLowerBound == 1

    DgpParams.gammaLowerBound = -0.95;

elseif negativeLowerBound == 0
    
    DgpParams.gammaLowerBound = 0;

end

DgpParams.gammaUpperBound = 0.95;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Compute support of the reduced form coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The support of the reduced form coefficients is used for defining grids
% over which we integrate pdfs for these coefficients

% See setDgpParams.m for further discussion
if DgpParams.gammaLowerBound >= 0
    %
    % pi_{12} = beta/(1-gamma1*gamma2)
    %    
    DgpParams.pi12lowerBound = ...
        DgpParams.betaZcoeff/(1-DgpParams.gammaLowerBound^2);
    
    DgpParams.pi12upperBound = ...
        DgpParams.betaZcoeff/(1-DgpParams.gammaUpperBound^2);
    
    %
    % pi_{22} = beta*gamma2/(1-gamma1*gamma2)
    %
    DgpParams.pi22lowerBound = ...
        DgpParams.betaZcoeff*DgpParams.gammaLowerBound/...
        (1-DgpParams.gammaLowerBound^2);
    
    DgpParams.pi22upperBound = ...
        DgpParams.betaZcoeff*DgpParams.gammaUpperBound/...
        (1-DgpParams.gammaUpperBound^2);
    
    %
    % pi_{11} = (U1 + gamma1*U2)/(1-gamma1*gamma2)
    %
    
    UsupportLowerBound = -2;
    UsupportUpperBound = 2;
    
    % Here I'm assuming that the support of U1 and U2 is
    %
    % (-UsupportLowerBound,UsupportUpperBound) + DgpParams.intercept
    %
    % For example, this is essentially the case if they are 
    % independent N(0,1)
    DgpParams.pi11lowerBound = ...
        ((UsupportLowerBound+DgpParams.intercept)...
        +(UsupportLowerBound+DgpParams.intercept)...
        *DgpParams.gammaUpperBound)/...
        (1-DgpParams.gammaUpperBound^2);

    if UsupportUpperBound + DgpParams.intercept < 0
        
        DgpParams.pi11upperBound = ...
            ((UsupportUpperBound+DgpParams.intercept)...
            +(UsupportUpperBound+DgpParams.intercept)...
            *DgpParams.gammaLowerBound)/...
            (1-DgpParams.gammaLowerBound^2);
    
    elseif UsupportUpperBound + DgpParams.intercept > 0

        DgpParams.pi11upperBound = ...
            ((UsupportUpperBound+DgpParams.intercept)...
            +(UsupportUpperBound+DgpParams.intercept)...
            *DgpParams.gammaUpperBound)/...
            (1-DgpParams.gammaUpperBound^2); 
    
    end
     
    %
    % pi_{21} = (U2 + gamma2*U1)/(1-gamma1*gamma2)
    %
    DgpParams.pi21lowerBound = DgpParams.pi11lowerBound;
    DgpParams.pi21upperBound = DgpParams.pi11upperBound;
    
    %
    % pi_{13} = gamma1*beta/(1-gamma1*gamma2)
    %
    DgpParams.pi13lowerBound = DgpParams.pi22lowerBound;
    DgpParams.pi13upperBound = DgpParams.pi22upperBound;
    
    %
    % pi_{23} = beta/(1-gamma1*gamma2)
    %
    DgpParams.pi23lowerBound = DgpParams.pi12lowerBound;
    DgpParams.pi23upperBound = DgpParams.pi12upperBound;
    
elseif DgpParams.gammaLowerBound < 0 && DgpParams.gammaUpperBound > 0
 
    % This case is not necessary
    error('Support of gamma computations incomplete');
    
elseif DgpParams.gammaUpperBound <= 0
    
    % This case is not necessary
    error('Support of gamma computations incomplete');
    
end