%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% setGridParameters.m
%
% AUTHOR: Margaux Luflade and Matthew A. Masten
%
% DATE CREATED: Feb 25, 2016
% DATE LAST UPDATED: July 14, 2017
%
% REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
% Simultaneous Equations Models", Review of Economic Studies
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function GridParams = setGridParameters(dgpChoice,DgpParams)
%% Dimension of first stage regressors
GridParams.covariateDimension = 3;

%% T grid

% Paramters for (t1,t2) grid
GridParams.nTgrid = 3500;

% These bounds correspond to the parameter M in the paper
% Remember the inverse scale relationship between the scale of a function
% and the scale of its Fourier transform. The tighter the function the
% larger its Fourier transform. Hence if we squeeze the data distributions
% then we need to increase the grid over which we are integrating its
% Fourier transform
if dgpChoice == 3
    M = 4;
elseif dgpChoice == 5
    M = 8;
elseif dgpChoice == 7
    M = 30;
else
    M = 10;
end

GridParams.TgridLowerBound = -M;
GridParams.TgridUpperBound = M;

%% G2 grid

% Parameters for integration in computing the analytical density in the
% first stage. Not needed if the estimated density is used instead
GridParams.nG2grid = 500;

%% S grid

% For integration in step (2) where we integrate over f_{Pi2(t1,t2)}(s)
% to get the characteristic function of (pi12,pi22)
GridParams.nSgrid = 40;

%% controlVarsGrid

GridParams.nControlVarsGrid = 50;

%% Z grid

% Grid of points z which we will evaluate f_{\gamma_2}(z) at.
% This grid defines what our final output will look like, because the pdf
% f_{\gamma_2}(z) is what we are ultimately interested in.
%
% This is a one dimensional plot over the support of \gamma_2, which is
% bounded.
%
% We will use a uniform grid. So all we need to set is the number of grid
% points and the upper and lower bound of the support of \gamma_2.

GridParams.nZgrid = 40; % Number of grid points

%% V grid

% The pdf of gamma_2 is a ratio pdf. Following Curtis (1941) [see Masten
% (2017) for citation] this ratio pdf is written as an integral of the
% joint pdf. Hence we need to choose a grid for numerically computing this
% integral. This is a one dimensional grid.
%
% Here we pick the grid of points v over which we will evaluate
% f_{\pi_{22},\pi_{12}(zv,v)
%
% Because this is a one dimensional integrand, we use a uniform grid.
% So we only need to pick the number of grid points and the lower and upper
% bounds of the grid.

GridParams.nVgrid = 50;
% The grid for V should cover the support of
%
% pi_{12} = beta/(1-gamma1*gamma2)
%
GridParams.VgridLowerBound = DgpParams.pi12lowerBound;
GridParams.VgridUpperBound = DgpParams.pi12upperBound;

% Important note: Although the above gives the logical support, the
% approximation performs poorly when you include the entire support.
% This is likely because of the |v| term, which amplifies errors in
% approximations in the upper tail of the pdf. This is basically the same
% problem as with the inverse Radon transform, which also has an absolute
% value entering in a similar way.
%
% Therfore a much more stable approach is to truncate it, as below.
% Strictly speaking this regularization is not theoretically necessary when
% the support of the coefficients is bounded, but it is nonetheless helpful
% in finite samples.

if dgpChoice == 1
    GridParams.VgridLowerBound = 5;
    GridParams.VgridUpperBound = 7;    
    
elseif dgpChoice == 3
    GridParams.VgridLowerBound = 5;
    GridParams.VgridUpperBound = 16;
    
elseif dgpChoice == 5
    GridParams.VgridLowerBound = 5;
    GridParams.VgridUpperBound = 14;

elseif dgpChoice == 7    
    GridParams.VgridLowerBound = 0;
    GridParams.VgridUpperBound = 3;
   
end

end
