// AUTHOR: Matthew A. Masten
//
// DATE CREATED: August 12, 2012
// DATE LAST UPDATED: July 14, 2017
//
// REFERENCE: Masten (2017) "Random Coefficients on Endogenous Variables in
// Simultaneous Equations Models", Review of Economic Studies

/* DESCRIPTION:
 I create two kinds of datasets: a singles and a pairs dataset. The pairs dataset is what we ultimately want. 
The singles dataset is just used to help keep track of which AIDs we've matched already.
*/

// So, since we want to give priority to an H1H2 pair over an H2H2 pair.
// Ok. so there are two options:
// 1. Remove all ``same'' pairs---H1H1, H2H2, H3H3, etc. first. Then look at H1H2, H1H3, etc.
// 2. Remove H1H1. Then remove H2H2 *only* from the using dataset, and look at H1H2.
//
// Actually, as long as each person did not nominate the same friend twice (within the in-home survey---obviously
// there could be double nominations across the in-home and in-school), then both approaches should yield the same thing.

/* Algorithm:
Always work with two datasets here, the master and the using. Call the ID used for merge in the relevant
datasets the masterID and usingID
We will loop over all permutations of the 10 different nomination sets.
1. Set masterID to pairID_IHMF1.
2. Set usingID to pairID_IHMF1.
3. Match. There will be duplicates. Flag the matches and remove them
4. Set usingID to pairID_IHMF2.
5. Match.
...
-Set master ID to pairID_IHMF2. Set usingID to pairID_IHMF2.
Note that the usingID is always `behind' the masterID.
*/

// i  is the value of the master dataset
// j is the value of the using dataset
////////////////////////////////////////////////////////////////////////////////

foreach gender in M F {

cd $data_cr_data_datasets_filepath
use friendNominations_`gender'_withIDs.dta, clear
tempfile friendData // This dataset is `emptied' as the code runs
save `friendData'

forvalues i = 1/5 {
forvalues j = 1/5 {
if `i' <= `j' {
if `i' == `j' {

// Here we pair people based on IH`i' alone

// 1. Make a singles dataset
gen mergingID = pairID_IH`gender'F`i' // We will look for observations who named each other as friends via IH`gender'F1
drop if mergingID == "" // Drop anyone who didn't nominate someone via IHMF; obviously they can't be paired
keep AID mergingID

gen pairPosition = .
bys mergingID: replace pairPosition = _n
egen tempMatch = max(pairPosition), by(mergingID)
drop if tempMatch == 1 // Drop anyone who was not matched
gen pairDef = "H`i'" // Define variable which tells us how this match was made. Here (unlike in the steps later on) it is the same for both people in the pair.
drop tempMatch

preserve
drop pairPosition
save `gender'singlesH`i'H`i'.dta, replace
restore

// 2. Convert the singles dataset to a pairs dataset
// We split the observations in half and then merge them together
preserve
tempfile usingData
drop if pairPosition == 1
rename AID AID1
rename pairDef pairDef_1
drop pairPosition
save `usingData'

restore
drop if pairPosition == 2
rename AID AID2
rename pairDef pairDef_2
drop pairPosition
merge mergingID using `usingData'
drop _merge

save `gender'pairsH`i'H`i'.dta, replace

// 3. Delete people who were matched by this step from the master dataset
use `friendData', clear

merge AID using `gender'singlesH`i'H`i'.dta, sort
drop if _merge == 3 // These are the people who are matched already
drop _merge pairDef mergingID

save `friendData', replace
// Note that pairID_IH`gender'F`i' is now a unique identifier, since we have removed any people who are paired
// by this variable. But "pairID_IH`gender'F`i'+1" is not necessarily a unique identifier.

}
else {

// Here we pair people based on IH`i' and IH`j'

use `friendData', clear

// 1. Make the pairs dataset
preserve
tempfile usingData
gen mergingID = pairID_IH`gender'F`j'
drop if mergingID == ""
keep AID mergingID
rename AID AID2
save `usingData'

restore
gen mergingID = pairID_IH`gender'F`i'
drop if mergingID == ""
keep AID mergingID
rename AID AID1

merge mergingID using `usingData', sort uniqmaster

keep if _merge == 3
drop _merge
gen pairDef_1 = "H`i'" // Variables to keep track of how exactly these two people were paired
gen pairDef_2 = "H`j'"

save `gender'pairsH`i'H`j'.dta, replace

// 2. Convert the pairs dataset to a singles dataset
preserve
tempfile usingData
drop AID2 pairDef_2
rename AID1 AID
rename pairDef_1 pairDef
save `usingData'

restore
drop AID1 pairDef_1
rename AID2 AID
rename pairDef_2 pairDef
append using `usingData'
save `gender'singlesH`i'H`j'.dta, replace

// 3. Ensure that people are uniquely matched
// The procedure above may result in the same person being matched to more than one person.
// Indeed, this actually happens in the dataset.
// For example, suppose 
//
// A nominates B in IH1
// B nominates A in IH2
//
// C nominates A in IH1
// A nominates C in IH2
//
// The above data pattern results in the pairs (A,C) and (A,B).
//
// One possibility is to drop both of these pairs.
// Alternatively, we can arbitrarily pick one of them to include. This is what I do.
// Also note that it is not possible for a single person to appear in three matches during this step.

gen tempMatch = .
bys AID: replace tempMatch = _n // Now we need to delete all mergingID's with tempMatch _n = 2
egen tempMatch2 = max(tempMatch), by(mergingID)
drop if tempMatch2 == 2 // Quasi-arbitrary drop of duplicate matches. [We're not randomly picking which to drop though]
drop tempMatch tempMatch2
save `gender'singlesH`i'H`j'.dta, replace

// 4. Update the pairs dataset.

gen pairPosition = .
bys mergingID: replace pairPosition = _n

preserve
tempfile usingData
drop if pairPosition == 1
rename AID AID1
rename pairDef pairDef_1
drop pairPosition
save `usingData'

restore
drop if pairPosition == 2
rename AID AID2
rename pairDef pairDef_2
drop pairPosition
merge mergingID using `usingData'
drop _merge

save `gender'pairsH`i'H`j'.dta, replace

// 5. Delete people who were matched by this step from the master dataset
use `friendData', clear
merge AID using `gender'singlesH`i'H`j'.dta, sort
drop if _merge == 3 // These are the people who are matched already
drop _merge pairDef mergingID
save `friendData', replace

}

}
}
}

use `friendData'
save friendNominations_`gender'_withIDs_unmatched.dta, replace

/////////////////////////////////////////////////////////////////////////////////////
// Next I append all of the pairs datasets together to create one master pairs dataset.

use `gender'pairsH1H1.dta, clear

forvalues i = 1/5 {
forvalues j = 1/5 {
if `i' <= `j' {
if `i' != 1 | `j' != 1 {
// Make sure we don't append the H1H1 dataset that we start with!

append using `gender'pairsH`i'H`j'

}
}
}
}

label variable pairDef_1 "How ego nominated alter" // E.g "H1" means ego nominated alter as 1st ranked friend in the in-home survey
label variable pairDef_2 "How ego nominated alter"
rename mergingID pairID
label variable pairID "Unique pair ID"

cd $data_datasets_filepath
save `gender'pairsIH.dta, replace

}

/////////////////////////////////////////////////////////////////////////////////////
// Append the male and female datasets together

use MpairsIH.dta, clear
append using FpairsIH
save pairsIH.dta, replace
